@@ -10,6 +10,7 @@ from warnings import warn
 
 from .base import REGISTRY, Backend, BaseBackend, BasePyExtBackend, from_name, register
 from .cuda_backend import CudaBackend
+from .debug_backend import DebugBackend
 from .gtcpp_backend import GTCpuIfirstBackend, GTCpuKfirstBackend, GTGpuBackend
 from .module_generator import BaseModuleGenerator
 from .numpy_backend import NumpyBackend
@@ -22,6 +23,7 @@ __all__ = [
     "BaseModuleGenerator",
     "BasePyExtBackend",
     "CudaBackend",
+    "DebugBackend",
     "GTCpuIfirstBackend",
     "GTCpuKfirstBackend",
     "GTGpuBackend",
@@ -32,11 +34,11 @@ __all__ = [
 
 
 try:
-    from .dace_backend import DaceCPUBackend, DaceGPUBackend
+    from .dace_backend import DaceCPUBackend, DaceCPUKFirstBackend, DaceGPUBackend
 
-    __all__ += ["DaceCPUBackend", "DaceGPUBackend"]
+    __all__ += ["DaceCPUBackend", "DaceCPUKFirstBackend", "DaceGPUBackend"]
 except ImportError:
     warn(
-        "GT4Py was unable to load DaCe. DaCe backends (`dace:cpu` and `dace:gpu`) will not be available.",
+        "GT4Py was unable to load DaCe. DaCe backends (`dace:cpu`, `dace:cpu_kfirst`, and `dace:gpu`) will not be available.",
         stacklevel=2,
     )
