@@ -16,7 +16,9 @@ from typing import TYPE_CHECKING, ClassVar
 
 from dace import SDFG, Memlet, SDFGState, config, data, dtypes, nodes, subsets, symbolic
 from dace.codegen import codeobject
+from dace.sdfg.analysis.schedule_tree import treenodes as tn
 from dace.sdfg.utils import inline_sdfgs
+from dace.transformation.passes import SimplifyPass
 
 from gt4py._core import definitions as core_defs
 from gt4py.cartesian import config as gt_config, definitions
@@ -31,18 +33,15 @@ from gt4py.cartesian.backend.gtc_common import (
     pybuffer_to_sid,
 )
 from gt4py.cartesian.backend.module_generator import make_args_data_from_gtir
-from gt4py.cartesian.gtc import common, gtir
-from gt4py.cartesian.gtc.dace import daceir as dcir
-from gt4py.cartesian.gtc.dace.nodes import StencilComputation
-from gt4py.cartesian.gtc.dace.oir_to_dace import OirSDFGBuilder
-from gt4py.cartesian.gtc.dace.transformations import (
-    NoEmptyEdgeTrivialMapElimination,
-    nest_sequential_map_scopes,
-)
+from gt4py.cartesian.gtc import gtir
+from gt4py.cartesian.gtc.dace import passes
+from gt4py.cartesian.gtc.dace.oir_to_treeir import OIRToTreeIR
+from gt4py.cartesian.gtc.dace.treeir_to_stree import TreeIRToScheduleTree
 from gt4py.cartesian.gtc.dace.utils import array_dimensions, replace_strides
+from gt4py.cartesian.gtc.definitions import CartesianSpace
 from gt4py.cartesian.gtc.gtir_to_oir import GTIRToOIR
 from gt4py.cartesian.gtc.passes.gtir_k_boundary import compute_k_boundary
-from gt4py.cartesian.gtc.passes.gtir_pipeline import GtirPipeline
+from gt4py.cartesian.gtc.passes.oir_optimizations import caches
 from gt4py.cartesian.gtc.passes.oir_optimizations.utils import compute_fields_extents
 from gt4py.cartesian.gtc.passes.oir_pipeline import DefaultPipeline
 from gt4py.cartesian.utils import shash
@@ -56,132 +55,36 @@ if TYPE_CHECKING:
     from gt4py.cartesian.stencil_object import StencilObject
 
 
-def _specialize_transient_strides(sdfg: SDFG, layout_info: layout.LayoutInfo) -> None:
-    replacement_dictionary = replace_strides(
-        [array for array in sdfg.arrays.values() if array.transient], layout_info["layout_map"]
+def _specialize_transient_strides(
+    sdfg: SDFG, layout_info: layout.LayoutInfo, replacement_dictionary: dict[str, str] | None = None
+) -> None:
+    # Find transients in this SDFG to specialize.
+    stride_replacements = replace_strides(
+        [
+            array
+            for array in sdfg.arrays.values()
+            if isinstance(array, data.Array) and array.transient
+        ],
+        layout_info,
     )
+
+    # In case of nested SDFGs (see below), merge with replacement dict that was passed down.
+    # Dev note: We shouldn't use mutable data structures as argument defaults.
+    replacement_dictionary = {} if replacement_dictionary is None else replacement_dictionary
+    replacement_dictionary.update(stride_replacements)
+
+    # Replace in this SDFG
     sdfg.replace_dict(replacement_dictionary)
     for state in sdfg.nodes():
         for node in state.nodes():
             if isinstance(node, nodes.NestedSDFG):
-                for k, v in replacement_dictionary.items():
-                    if k in node.symbol_mapping:
-                        node.symbol_mapping[k] = v
+                # Recursively replace strides in nested SDFGs
+                _specialize_transient_strides(node.sdfg, layout_info, replacement_dictionary)
     for k in replacement_dictionary.keys():
         if k in sdfg.symbols:
             sdfg.remove_symbol(k)
 
 
-def _get_expansion_priority_cpu(node: StencilComputation):
-    expansion_priority = []
-    if node.has_splittable_regions():
-        expansion_priority.append(["Sections", "Stages", "I", "J", "K"])
-    expansion_priority.extend(
-        [
-            ["TileJ", "TileI", "IMap", "JMap", "Sections", "K", "Stages"],
-            ["TileJ", "TileI", "IMap", "JMap", "Sections", "Stages", "K"],
-            ["TileJ", "TileI", "Sections", "Stages", "IMap", "JMap", "K"],
-            ["TileJ", "TileI", "Sections", "K", "Stages", "JMap", "IMap"],
-        ]
-    )
-    return expansion_priority
-
-
-def _get_expansion_priority_gpu(node: StencilComputation):
-    expansion_priority = []
-    if node.has_splittable_regions():
-        expansion_priority.append(["Sections", "Stages", "J", "I", "K"])
-    if node.oir_node.loop_order == common.LoopOrder.PARALLEL:
-        expansion_priority.append(["Sections", "Stages", "K", "J", "I"])
-    else:
-        expansion_priority.append(["J", "I", "Sections", "Stages", "K"])
-    expansion_priority.append(["TileJ", "TileI", "Sections", "K", "Stages", "JMap", "IMap"])
-    return expansion_priority
-
-
-def _set_expansion_orders(sdfg: SDFG):
-    for node, _ in filter(
-        lambda n: isinstance(n[0], StencilComputation), sdfg.all_nodes_recursive()
-    ):
-        if node.device == dtypes.DeviceType.GPU:
-            expansion_priority = _get_expansion_priority_gpu(node)
-        else:
-            expansion_priority = _get_expansion_priority_cpu(node)
-        is_set = False
-        for exp in expansion_priority:
-            try:
-                node.expansion_specification = exp
-                is_set = True
-            except ValueError:
-                continue
-            else:
-                break
-        if not is_set:
-            raise ValueError("No expansion compatible")
-
-
-def _set_tile_sizes(sdfg: SDFG):
-    for node, _ in filter(
-        lambda n: isinstance(n[0], StencilComputation), sdfg.all_nodes_recursive()
-    ):
-        if node.device == dtypes.DeviceType.GPU:
-            node.tile_sizes = {dcir.Axis.I: 64, dcir.Axis.J: 8, dcir.Axis.K: 8}
-            node.tile_sizes_interpretation = "shape"
-        else:
-            node.tile_sizes = {dcir.Axis.I: 8, dcir.Axis.J: 8, dcir.Axis.K: 8}
-            node.tile_sizes_interpretation = "strides"
-
-
-def _to_device(sdfg: SDFG, device: str) -> None:
-    """Update sdfg in place."""
-    if device == "gpu":
-        for array in sdfg.arrays.values():
-            array.storage = dtypes.StorageType.GPU_Global
-        for node, _ in sdfg.all_nodes_recursive():
-            if isinstance(node, StencilComputation):
-                node.device = dtypes.DeviceType.GPU
-
-
-def _pre_expand_transformations(
-    gtir_pipeline: GtirPipeline, sdfg: SDFG, layout_info: layout.LayoutInfo
-):
-    args_data = make_args_data_from_gtir(gtir_pipeline)
-
-    # stencils without effect
-    if all(info is None for info in args_data.field_info.values()):
-        sdfg = SDFG(gtir_pipeline.gtir.name)
-        sdfg.add_state(gtir_pipeline.gtir.name)
-        return sdfg
-
-    sdfg.simplify(validate=False)
-
-    _set_expansion_orders(sdfg)
-    _set_tile_sizes(sdfg)
-    _specialize_transient_strides(sdfg, layout_info)
-    return sdfg
-
-
-def _post_expand_transformations(sdfg: SDFG):
-    # DaCe "standard" clean-up transformations
-    sdfg.simplify(validate=False)
-
-    sdfg.apply_transformations_repeated(NoEmptyEdgeTrivialMapElimination, validate=False)
-
-    # Control the `#pragma omp parallel` statements: Fully collapse parallel loops,
-    # but set 1D maps to be sequential. (Typical domains are too small to benefit from parallelism)
-    for node, _ in filter(lambda n: isinstance(n[0], nodes.MapEntry), sdfg.all_nodes_recursive()):
-        node.collapse = len(node.range)
-        if node.schedule == dtypes.ScheduleType.CPU_Multicore and len(node.range) <= 1:
-            node.schedule = dtypes.ScheduleType.Sequential
-
-    # To be re-evaluated with https://github.com/GridTools/gt4py/issues/1896
-    # sdfg.apply_transformations_repeated(InlineThreadLocalTransients, validate=False) # noqa: ERA001
-    sdfg.simplify(validate=False)
-    nest_sequential_map_scopes(sdfg)
-    for sd in sdfg.all_sdfgs_recursive():
-        sd.openmp_sections = False
-
-
 def _sdfg_add_arrays_and_edges(
     field_info: dict[str, definitions.FieldInfo],
     wrapper_sdfg: SDFG,
@@ -190,10 +93,13 @@ def _sdfg_add_arrays_and_edges(
     nsdfg: nodes.NestedSDFG,
     inputs: set[str] | dict[str, dtypes.typeclass],
     outputs: set[str] | dict[str, dtypes.typeclass],
-    origins,
+    origins: dict[str, tuple[int, ...]],
 ) -> None:
     for name, array in inner_sdfg.arrays.items():
-        if isinstance(array, data.Array) and not array.transient:
+        if array.transient:
+            continue
+
+        if isinstance(array, data.Array):
             axes = field_info[name].axes
 
             shape = [f"__{name}_{axis}_size" for axis in axes] + [
@@ -201,22 +107,41 @@ def _sdfg_add_arrays_and_edges(
             ]
 
             wrapper_sdfg.add_array(
-                name, dtype=array.dtype, strides=array.strides, shape=shape, storage=array.storage
+                name,
+                dtype=array.dtype,
+                strides=array.strides,
+                shape=shape,
+                storage=array.storage,
             )
-            if isinstance(origins, tuple):
-                origin = [o for a, o in zip("IJK", origins) if a in axes]
-            else:
-                origin = origins.get(name, origins.get("_all_", None))
-                if len(origin) == 3:
-                    origin = [o for a, o in zip("IJK", origin) if a in axes]
 
-            ranges = [
-                (o - max(0, e), o - max(0, e) + s - 1, 1)
-                for o, e, s in zip(
-                    origin, field_info[name].boundary.lower_indices, inner_sdfg.arrays[name].shape
+            # Calculate memlet ranges taking the origin into account
+            ranges = []
+            origin = origins.get(name, origins.get("_all_", None))
+
+            if origin is None:
+                raise ValueError("Freeze origin/domain: Unspecified origin for field `{name}`.")
+
+            # Read boundaries for axis-bound fields
+            index = 0
+            for cartesian_index, axis in enumerate(CartesianSpace.names):
+                if axis not in axes:
+                    continue
+                o = origin[index]
+                e = field_info[name].boundary.lower_indices[cartesian_index]
+                s = inner_sdfg.arrays[name].shape[index]
+                ranges.append(
+                    # s - 1 because ranges are inclusive
+                    (o - max(0, e), o - max(0, e) + s - 1, 1)
                 )
+                index += 1
+
+            # Add data dimensions to the range
+            # NOTE: origin and boundary of data dimensions are always 0
+            ranges += [
+                (0, d - 1, 1)  # d - 1 because ranges are inclusive
+                for d in field_info[name].data_dims
             ]
-            ranges += [(0, d, 1) for d in field_info[name].data_dims]
+
             if name in inputs:
                 state.add_edge(
                     state.add_read(name),
@@ -233,6 +158,29 @@ def _sdfg_add_arrays_and_edges(
                     None,
                     Memlet(name, subset=subsets.Range(ranges)),
                 )
+        elif isinstance(array, data.Scalar):
+            wrapper_sdfg.add_scalar(
+                name,
+                dtype=array.dtype,
+                storage=array.storage,
+                lifetime=array.lifetime,
+            )
+            if name in inputs:
+                state.add_edge(
+                    state.add_read(name),
+                    None,
+                    nsdfg,
+                    name,
+                    Memlet(name),
+                )
+            if name in outputs:
+                state.add_edge(
+                    nsdfg,
+                    name,
+                    state.add_write(name),
+                    None,
+                    Memlet(name),
+                )
 
 
 def _sdfg_specialize_symbols(wrapper_sdfg: SDFG, domain: tuple[int, ...]) -> None:
@@ -272,13 +220,37 @@ def _sdfg_specialize_symbols(wrapper_sdfg: SDFG, domain: tuple[int, ...]) -> Non
 
 
 def freeze_origin_domain_sdfg(
-    inner_sdfg: SDFG,
+    inner_sdfg_unfrozen: SDFG,
     arg_names: list[str],
     field_info: dict[str, definitions.FieldInfo],
     *,
+    layout_info: layout.LayoutInfo,
     origin: dict[str, tuple[int, ...]],
     domain: tuple[int, ...],
 ) -> SDFG:
+    """Create a new SDFG by wrapping a _copy_ of the original SDFG and freezing it's
+    origin and domain.
+
+    This wrapping is required because we do not expect any of the inner_sdfg bounds to
+    have been specialized, e.g. we expect "__I/J/K" symbols to still be present. We wrap
+    the call and specialize at top level, which will then be passed as a symbol to the
+    inner sdfg.
+
+    Once we move specialization of array & maps bounds upstream, this will become moot
+    and can be removed, see https://github.com/GridTools/gt4py/issues/2082.
+
+    Dev note: we need to wrap a copy to make sure we can use caching with no side effects
+    in other parts of the SDFG making pipeline.
+
+    Args:
+        inner_sdfg_unfrozen: SDFG with cartesian bounds as symbols
+        arg_names: names of arguments to freeze
+        field_info: full info stack on arguments
+        origin: tuple of offset into the memory
+        domain: tuple of size for the memory written by the stencil
+    """
+    inner_sdfg = copy.deepcopy(inner_sdfg_unfrozen)
+
     wrapper_sdfg = SDFG("frozen_" + inner_sdfg.name)
     state = wrapper_sdfg.add_state("frozen_" + inner_sdfg.name + "_state")
 
@@ -296,7 +268,7 @@ def freeze_origin_domain_sdfg(
     nsdfg = state.add_nested_sdfg(inner_sdfg, None, inputs, outputs)
 
     _sdfg_add_arrays_and_edges(
-        field_info, wrapper_sdfg, state, inner_sdfg, nsdfg, inputs, outputs, origins=origin
+        field_info, wrapper_sdfg, state, inner_sdfg, nsdfg, inputs, outputs, origin
     )
 
     # in special case of empty domain, remove entire SDFG.
@@ -315,6 +287,7 @@ def freeze_origin_domain_sdfg(
     inline_sdfgs(wrapper_sdfg)
 
     _sdfg_specialize_symbols(wrapper_sdfg, domain)
+    _specialize_transient_strides(wrapper_sdfg, layout_info)
 
     for _, _, array in wrapper_sdfg.arrays_recursive():
         if array.transient:
@@ -326,11 +299,53 @@ def freeze_origin_domain_sdfg(
 
 
 class SDFGManager:
-    # Cache loaded SDFGs across all instances
+    # Cache loaded SDFGs across all instances (unless caching strategy is "nocaching")
     _loaded_sdfgs: ClassVar[dict[str | pathlib.Path, SDFG]] = dict()
 
-    def __init__(self, builder: StencilBuilder) -> None:
+    def __init__(self, builder: StencilBuilder, debug_stree: bool = False) -> None:
+        """
+        Initializes the SDFGManager.
+
+        Args:
+          builder: The StencilBuilder instance, used for build options and caching strategy.
+          debug_stree: If true, saves a string representation of the schedule tree next to the cached SDFG.
+        """
         self.builder = builder
+        self.debug_stree = debug_stree
+
+    def schedule_tree(self) -> tn.ScheduleTreeRoot:
+        """
+        Schedule tree representation of the gtir (taken from the builder).
+
+        This function is a three-step process:
+
+        oir = gtir_to_oir(self.builder.gtir)
+        tree_ir = oir_to_tree_ir(oir)
+        schedule_tree = tree_ir_to_schedule_tree(tree_ir)
+        """
+
+        oir = GTIRToOIR().visit(self.builder.gtir)
+
+        # Deactivate caches. We need to extend the skip list in case users have
+        # specified skip as well AND we need to copy in order to not trash the
+        # cache hash!
+        oir_pipeline: DefaultPipeline = self.builder.options.backend_opts.get(
+            "oir_pipeline", DefaultPipeline()
+        )
+        oir_pipeline = copy.deepcopy(oir_pipeline)
+        oir_pipeline.skip.extend(
+            [
+                caches.IJCacheDetection,
+                caches.KCacheDetection,
+                caches.PruneKCacheFills,
+                caches.PruneKCacheFlushes,
+            ]
+        )
+        oir = oir_pipeline.run(oir)
+
+        tir = OIRToTreeIR(self.builder).visit(oir)
+
+        return TreeIRToScheduleTree().visit(tir)
 
     @staticmethod
     def _strip_history(sdfg: SDFG) -> None:
@@ -340,71 +355,91 @@ class SDFGManager:
             tmp_sdfg.orig_sdfg = None
 
     @staticmethod
-    def _save_sdfg(sdfg: SDFG, path: str) -> None:
+    def _save_sdfg(sdfg: SDFG, path: pathlib.Path, validate: bool = False) -> None:
+        if validate:
+            sdfg.validate()
         SDFGManager._strip_history(sdfg)
-        sdfg.save(path)
+        sdfg.save(str(path))
 
-    def _unexpanded_sdfg(self):
-        filename = self.builder.module_name + ".sdfg"
+    def sdfg_via_schedule_tree(self, *, validate: bool = True, simplify: bool = True) -> SDFG:
+        """Lower OIR into an SDFG via Schedule Tree transpile first.
+
+        Cache the SDFG into the manager for re-use, unless the builder has a no-caching policy.
+
+        Args:
+            validate: Validate resulting SDFG
+            simplify: Simplify resulting SDFG
+        """
+        filename = f"{self.builder.module_name}.sdfg"
         path = (
             pathlib.Path(os.path.relpath(self.builder.module_path.parent, pathlib.Path.cwd()))
             / filename
         )
 
-        if path not in SDFGManager._loaded_sdfgs:
-            try:
+        do_cache = self.builder.caching.name != "nocaching"
+        if do_cache:
+            if path in SDFGManager._loaded_sdfgs:
+                return SDFGManager._loaded_sdfgs[path]
+            if path.exists():
                 sdfg = SDFG.from_file(path)
-            except FileNotFoundError:
-                base_oir = GTIRToOIR().visit(self.builder.gtir)
-                oir_pipeline = self.builder.options.backend_opts.get(
-                    "oir_pipeline", DefaultPipeline()
-                )
-                oir_node = oir_pipeline.run(base_oir)
-                sdfg = OirSDFGBuilder().visit(oir_node)
+                SDFGManager._loaded_sdfgs[path] = sdfg
+                return sdfg
 
-                _to_device(sdfg, self.builder.backend.storage_info["device"])
-                _pre_expand_transformations(
-                    self.builder.gtir_pipeline, sdfg, self.builder.backend.storage_info
-                )
-                self._save_sdfg(sdfg, path)
+        # Create SDFG
+        stree = self.schedule_tree()
+
+        if self.builder.backend.name.endswith("_kfirst"):
+            # re-order loops to match loops with memory layout
+            flipper = passes.PushVerticalMapDown()
+            flipper.visit(stree)
+
+        sdfg = stree.as_sdfg(
+            validate=validate,
+            simplify=simplify,
+            skip={"ScalarToSymbolPromotion"},
+        )
+
+        if do_cache:
+            self._save_sdfg(sdfg, path)
             SDFGManager._loaded_sdfgs[path] = sdfg
 
-        return SDFGManager._loaded_sdfgs[path]
+            if self.debug_stree:
+                stree_path = path.with_suffix(".stree.txt")
+                with open(stree_path, "w+") as file:
+                    file.write(stree.as_string(-1))
 
-    def unexpanded_sdfg(self):
-        return copy.deepcopy(self._unexpanded_sdfg())
-
-    def _expanded_sdfg(self):
-        sdfg = self._unexpanded_sdfg()
-        sdfg.expand_library_nodes()
-        _post_expand_transformations(sdfg)
         return sdfg
 
-    def expanded_sdfg(self):
-        return copy.deepcopy(self._expanded_sdfg())
-
     def _frozen_sdfg(self, *, origin: dict[str, tuple[int, ...]], domain: tuple[int, ...]) -> SDFG:
         basename = self.builder.module_path.with_suffix("")
-        path = f"{basename}_{shash(origin, domain)}.sdfg"
+        path = pathlib.Path(f"{basename}_{shash(origin, domain)}.sdfg")
 
-        # check if same sdfg already cached on disk
-        if path in SDFGManager._loaded_sdfgs:
-            return SDFGManager._loaded_sdfgs[path]
+        # check if the same sdfg is already loaded
+        do_cache = self.builder.caching.name != "nocache"
+        if do_cache:
+            if path in SDFGManager._loaded_sdfgs:
+                return SDFGManager._loaded_sdfgs[path]
+            if path.exists():
+                sdfg = SDFG.from_file(path)
+                SDFGManager._loaded_sdfgs[path] = sdfg
+                return sdfg
 
-        # otherwise, wrap and save sdfg from scratch
-        inner_sdfg = self.unexpanded_sdfg()
-
-        sdfg = freeze_origin_domain_sdfg(
-            inner_sdfg,
+        # Otherwise, wrap and save sdfg from scratch
+        sdfg = self.sdfg_via_schedule_tree()
+        frozen_sdfg = freeze_origin_domain_sdfg(
+            sdfg,
             arg_names=[arg.name for arg in self.builder.gtir.api_signature],
             field_info=make_args_data_from_gtir(self.builder.gtir_pipeline).field_info,
+            layout_info=self.builder.backend.storage_info,
             origin=origin,
             domain=domain,
         )
-        SDFGManager._loaded_sdfgs[path] = sdfg
-        self._save_sdfg(sdfg, path)
 
-        return SDFGManager._loaded_sdfgs[path]
+        if do_cache:
+            SDFGManager._loaded_sdfgs[path] = frozen_sdfg
+            self._save_sdfg(frozen_sdfg, path)
+
+        return frozen_sdfg
 
     def frozen_sdfg(self, *, origin: dict[str, tuple[int, ...]], domain: tuple[int, ...]) -> SDFG:
         return copy.deepcopy(self._frozen_sdfg(origin=origin, domain=domain))
@@ -418,13 +453,22 @@ class DaCeExtGenerator(BackendCodegen):
 
     def __call__(self) -> dict[str, dict[str, str]]:
         manager = SDFGManager(self.backend.builder)
-        sdfg = manager.expanded_sdfg()
+
+        sdfg = manager.sdfg_via_schedule_tree()
+        _specialize_transient_strides(
+            sdfg,
+            self.backend.storage_info,
+        )
+        SimplifyPass(validate=True, skip={"ScalarToSymbolPromotion"}).apply_pass(sdfg, {})
+
+        # NOTE
+        # The glue code in DaCeComputationCodegen.apply() (just below) will define all the
+        # symbols. Our job creating the sdfg/stree is to make sure we use the same symbols
+        # and to be sure that these symbols are added as dace symbols.
 
         implementation = DaCeComputationCodegen.apply(self.backend.builder, sdfg)
 
-        bindings = DaCeBindingsCodegen.apply(
-            sdfg, module_name=self.module_name, backend=self.backend
-        )
+        bindings = DaCeBindingsCodegen.apply(sdfg, self.module_name, backend=self.backend)
 
         bindings_ext = "cu" if self.backend.storage_info["device"] == "gpu" else "cpp"
         return {
@@ -494,7 +538,7 @@ auto ${name}(const std::array<gt::uint_t, 3>& domain) {
         return res
 
     @staticmethod
-    def _postprocess_dace_code(code_objects: codeobject.CodeObject, is_gpu: bool) -> str:
+    def _postprocess_dace_code(code_objects: list[codeobject.CodeObject], is_gpu: bool) -> str:
         lines = code_objects[[co.title for co in code_objects].index("Frame")].clean_code.split(
             "\n"
         )
@@ -598,6 +642,10 @@ namespace gt = gridtools;
             if array.transient:
                 continue
 
+            if isinstance(array, data.Scalar):
+                # will be passed by name (as variable) by the catch all below
+                continue
+
             dims = [dim for dim, select in zip("IJK", array_dimensions(array)) if select]
             data_ndim = len(array.shape) - len(dims)
 
@@ -653,14 +701,20 @@ namespace gt = gridtools;
         for name, array in sdfg.arrays.items():
             if array.transient:
                 continue
-            arguments.append(f"auto && __{name}_sid")
+            if isinstance(array, data.Scalar):
+                arguments.append(f"auto {name}")
+                continue
+            if isinstance(array, data.Array):
+                arguments.append(f"auto && __{name}_sid")
+                continue
+            raise NotImplementedError(f"generate_functor_args(): unexpected type {type(array)}")
         for name, dtype in ((n, d) for n, d in sdfg.symbols.items() if not n.startswith("__")):
             arguments.append(dtype.as_arg(name))
         return arguments
 
 
 class DaCeBindingsCodegen:
-    def __init__(self, backend: BaseDaceBackend):
+    def __init__(self, backend: BaseDaceBackend) -> None:
         self.backend = backend
         self._unique_index: int = 0
 
@@ -676,16 +730,24 @@ class DaCeBindingsCodegen:
         for name in sdfg.signature_arglist(with_types=False, for_call=True):
             if name in sdfg.arrays:
                 container = sdfg.arrays[name]
-                assert isinstance(container, data.Array)
-                res[name] = (
-                    "py::{pybind_type} {name}, std::array<gt::int_t,{ndim}> {name}_origin".format(
-                        pybind_type=(
-                            "object" if self.backend.storage_info["device"] == "gpu" else "buffer"
-                        ),
-                        name=name,
-                        ndim=len(container.shape),
+                if isinstance(container, data.Scalar):
+                    res[name] = f"{container.ctype} {name}"
+                elif isinstance(container, data.Array):
+                    res[name] = (
+                        "py::{pybind_type} {name}, std::array<gt::int_t,{ndim}> {name}_origin".format(
+                            pybind_type=(
+                                "object"
+                                if self.backend.storage_info["device"] == "gpu"
+                                else "buffer"
+                            ),
+                            name=name,
+                            ndim=len(container.shape),
+                        )
+                    )
+                else:
+                    raise NotImplementedError(
+                        f"generate_entry_params(): unexpected type {type(container)}"
                     )
-                )
             elif name in sdfg.symbols and not name.startswith("__"):
                 res[name] = f"{sdfg.symbols[name].ctype} {name}"
         return list(res[node.name] for node in self.backend.builder.gtir.params if node.name in res)
@@ -697,10 +759,16 @@ class DaCeBindingsCodegen:
             if array.transient:
                 continue
 
+            if isinstance(array, data.Scalar):
+                res.append(name)
+                continue
+
+            if not isinstance(array, data.Array):
+                raise NotImplementedError(f"generate_sid_params(): unexpected type {type(array)}")
+
             domain_dim_flags = tuple(array_dimensions(array))
             if len(domain_dim_flags) != 3:
                 raise RuntimeError("Expected 3 cartesian array dimensions. Codegen error.")
-
             data_ndim = len(array.shape) - sum(domain_dim_flags)
             sid_def = pybuffer_to_sid(
                 name=name,
@@ -717,7 +785,7 @@ class DaCeBindingsCodegen:
             res.append(name)
         return res
 
-    def generate_sdfg_bindings(self, sdfg, module_name) -> str:
+    def generate_sdfg_bindings(self, sdfg: SDFG, module_name: str) -> str:
         return self.mako_template.render_values(
             name=sdfg.name,
             module_name=module_name,
@@ -727,7 +795,7 @@ class DaCeBindingsCodegen:
 
     @classmethod
     def apply(cls, sdfg: SDFG, module_name: str, *, backend: BaseDaceBackend) -> str:
-        generated_code = cls(backend).generate_sdfg_bindings(sdfg, module_name=module_name)
+        generated_code = cls(backend).generate_sdfg_bindings(sdfg, module_name)
         if backend.builder.options.format_source:
             generated_code = codegen.format_source("cpp", generated_code, style="LLVM")
         return generated_code
@@ -781,6 +849,24 @@ class BaseDaceBackend(BaseGTBackend):
 class DaceCPUBackend(BaseDaceBackend):
     name = "dace:cpu"
     languages: ClassVar[dict] = {"computation": "c++", "bindings": ["python"]}
+    storage_info: ClassVar[layout.LayoutInfo] = {
+        "alignment": 1,
+        "device": "cpu",
+        "layout_map": layout.layout_maker_factory((1, 2, 0)),
+        "is_optimal_layout": layout.layout_checker_factory(layout.layout_maker_factory((1, 2, 0))),
+    }
+    MODULE_GENERATOR_CLASS = DaCePyExtModuleGenerator
+
+    options = BaseGTBackend.GT_BACKEND_OPTS
+
+    def generate_extension(self) -> None:
+        return self.make_extension(uses_cuda=False)
+
+
+@register
+class DaceCPUKFirstBackend(BaseDaceBackend):
+    name = "dace:cpu_kfirst"
+    languages: ClassVar[dict] = {"computation": "c++", "bindings": ["python"]}
     storage_info: ClassVar[layout.LayoutInfo] = {
         "alignment": 1,
         "device": "cpu",
