@@ -20,6 +20,7 @@ from gt4py.cartesian.frontend.node_util import (
     location_to_source_location,
 )
 from gt4py.cartesian.frontend.nodes import (
+    AbsoluteKIndex,
     ArgumentInfo,
     Assign,
     AxisBound,
@@ -39,6 +40,7 @@ from gt4py.cartesian.frontend.nodes import (
     HorizontalIf,
     If,
     IterationOrder,
+    IteratorAccess,
     LevelMarker,
     NativeFuncCall,
     NativeFunction,
@@ -340,6 +342,14 @@ class DefIRToGTIR(IRNodeVisitor):
         NativeFunction.FLOOR: common.NativeFunction.FLOOR,
         NativeFunction.CEIL: common.NativeFunction.CEIL,
         NativeFunction.TRUNC: common.NativeFunction.TRUNC,
+        NativeFunction.INT32: common.NativeFunction.INT32,
+        NativeFunction.INT64: common.NativeFunction.INT64,
+        NativeFunction.FLOAT32: common.NativeFunction.FLOAT32,
+        NativeFunction.FLOAT64: common.NativeFunction.FLOAT64,
+        NativeFunction.ERF: common.NativeFunction.ERF,
+        NativeFunction.ERFC: common.NativeFunction.ERFC,
+        NativeFunction.ROUND: common.NativeFunction.ROUND,
+        NativeFunction.ROUND_AWAY_FROM_ZERO: common.NativeFunction.ROUND_AWAY_FROM_ZERO,
     }
 
     GT4PY_BUILTIN_TO_GTIR: Final[dict[Builtin, common.BuiltInLiteral]] = {
@@ -347,6 +357,19 @@ class DefIRToGTIR(IRNodeVisitor):
         Builtin.FALSE: common.BuiltInLiteral.FALSE,
     }
 
+    GT4PY_DTYPE_TO_GTIR_DTYPE: Final[dict[DataType, common.DataType]] = {
+        DataType.INVALID: common.DataType.INVALID,
+        DataType.AUTO: common.DataType.AUTO,
+        DataType.DEFAULT: common.DataType.DEFAULT,
+        DataType.BOOL: common.DataType.BOOL,
+        DataType.INT8: common.DataType.INT8,
+        DataType.INT16: common.DataType.INT16,
+        DataType.INT32: common.DataType.INT32,
+        DataType.INT64: common.DataType.INT64,
+        DataType.FLOAT32: common.DataType.FLOAT32,
+        DataType.FLOAT64: common.DataType.FLOAT64,
+    }
+
     @classmethod
     def apply(cls, root, **kwargs):
         return cls().visit(root)
@@ -406,6 +429,12 @@ class DefIRToGTIR(IRNodeVisitor):
             loc=location_to_source_location(node.loc),
         )
 
+    def visit_IteratorAccess(self, iterator_access: IteratorAccess) -> gtir.IteratorAccess:
+        return gtir.IteratorAccess(
+            name=gtir.IteratorAccess.AxisName(iterator_access.name),
+            dtype=self.GT4PY_DTYPE_TO_GTIR_DTYPE[iterator_access.data_type],
+        )
+
     def visit_BlockStmt(self, node: BlockStmt) -> List[gtir.Stmt]:
         return [self.visit(s) for s in node.stmts]
 
@@ -564,12 +593,16 @@ class DefIRToGTIR(IRNodeVisitor):
         )
 
     def transform_offset(
-        self, offset: Dict[str, Union[int, Expr]], **kwargs: Any
-    ) -> Union[common.CartesianOffset, gtir.VariableKOffset]:
+        self, offset: dict[str, int | Expr | AbsoluteKIndex], **kwargs: Any
+    ) -> common.CartesianOffset | gtir.VariableKOffset | gtir.AbsoluteKIndex:
+        if isinstance(offset, AbsoluteKIndex):
+            return gtir.AbsoluteKIndex(k=self.visit(offset.k, **kwargs))
+
         k_val = offset.get("K", 0)
         if isinstance(k_val, numbers.Integral):
             return common.CartesianOffset(i=offset.get("I", 0), j=offset.get("J", 0), k=k_val)
+
         if isinstance(k_val, Expr):
             return gtir.VariableKOffset(k=self.visit(k_val, **kwargs))
 
-        raise TypeError("Unrecognized vertical offset type")
+        raise TypeError("Unrecognized vertical offset type.")
