@@ -10,6 +10,7 @@ from __future__ import annotations
 
 import enum
 import functools
+import numbers
 import typing
 from typing import (
     Any,
@@ -178,6 +179,15 @@ class NativeFunction(eve.StrEnum):
     FLOOR = "floor"
     CEIL = "ceil"
     TRUNC = "trunc"
+    ERF = "erf"
+    ERFC = "erfc"
+    ROUND = "round"
+    ROUND_AWAY_FROM_ZERO = "round_away_from_zero"
+
+    INT32 = "int32"
+    INT64 = "int64"
+    FLOAT32 = "float32"
+    FLOAT64 = "float64"
 
     IR_OP_TO_NUM_ARGS: ClassVar[Dict[NativeFunction, int]]
 
@@ -218,6 +228,14 @@ NativeFunction.IR_OP_TO_NUM_ARGS = {
         NativeFunction.FLOOR: 1,
         NativeFunction.CEIL: 1,
         NativeFunction.TRUNC: 1,
+        NativeFunction.INT32: 1,
+        NativeFunction.INT64: 1,
+        NativeFunction.FLOAT32: 1,
+        NativeFunction.FLOAT64: 1,
+        NativeFunction.ERF: 1,
+        NativeFunction.ERFC: 1,
+        NativeFunction.ROUND: 1,
+        NativeFunction.ROUND_AWAY_FROM_ZERO: 1,
     }.items()
 }
 
@@ -332,6 +350,34 @@ class VariableKOffset(eve.GenericNode, Generic[ExprT]):
             raise ValueError("Variable vertical index must be an integer expression")
 
 
+class AbsoluteKIndex(eve.GenericNode, Generic[ExprT]):
+    """Access a field with absolute K
+
+    Restrictions:
+    - Centered I/J
+    - No data dimensions
+    - Read-only
+    """
+
+    k: Union[int, ExprT]
+
+    def to_dict(self) -> Dict[str, Optional[int]]:
+        return {"i": 0, "j": 0, "k": None}
+
+    @datamodels.validator("k")
+    def offset_expr_is_int(self, _attribute: datamodels.Attribute, value: Any) -> None:
+        utils.warn_experimental_feature(
+            feature="Absolute indexing in `K`", ADR="experimental/indexing-absolute-k.md"
+        )
+        if isinstance(value, numbers.Real):
+            if not isinstance(value, int):
+                raise ValueError("Absolute vertical index literal must be an integer")
+        else:
+            value = typing.cast(Expr, value)
+            if value.dtype is not DataType.AUTO and not value.dtype.isinteger():
+                raise ValueError("Absolute vertical index must be an integer expression")
+
+
 class ScalarAccess(LocNode):
     name: eve.Coerced[eve.SymbolRef]
     kind: ExprKind = ExprKind.SCALAR
@@ -339,7 +385,7 @@ class ScalarAccess(LocNode):
 
 class FieldAccess(eve.GenericNode, Generic[ExprT, VariableKOffsetT]):
     name: eve.Coerced[eve.SymbolRef]
-    offset: Union[CartesianOffset, VariableKOffsetT]
+    offset: CartesianOffset | VariableKOffsetT | AbsoluteKIndex
     data_index: List[ExprT] = eve.field(default_factory=list)
     kind: ExprKind = ExprKind.FIELD
 
@@ -549,9 +595,27 @@ class NativeFuncCall(eve.GenericNode, Generic[ExprT]):
 
 
 def native_func_call_dtype_propagation(*, strict: bool = True) -> datamodels.RootValidator:
+    def _precision_to_datatype(func: NativeFunction) -> DataType:
+        if func == NativeFunction.INT32:
+            return DataType.INT32
+        if func == NativeFunction.INT64:
+            return DataType.INT64
+        if func == NativeFunction.FLOAT32:
+            return DataType.FLOAT32
+        if func == NativeFunction.FLOAT64:
+            return DataType.FLOAT64
+        raise NotImplementedError(f"Found unknown precision specification {func}")
+
     def _impl(cls: Type[NativeFuncCall], instance: NativeFuncCall) -> None:
         if instance.func in (NativeFunction.ISFINITE, NativeFunction.ISINF, NativeFunction.ISNAN):
             instance.dtype = DataType.BOOL  # type: ignore[attr-defined]
+        elif instance.func in (
+            NativeFunction.INT32,
+            NativeFunction.INT64,
+            NativeFunction.FLOAT32,
+            NativeFunction.FLOAT64,
+        ):
+            instance.dtype = _precision_to_datatype(instance.func)  # type: ignore[attr-defined]
         else:
             # assumes all NativeFunction args have a common dtype
             common_dtype = verify_and_get_common_dtype(cls, instance.args, strict=strict)
@@ -887,6 +951,14 @@ OP_TO_UFUNC_NAME: Final[
         NativeFunction.FLOOR: "floor",
         NativeFunction.CEIL: "ceil",
         NativeFunction.TRUNC: "trunc",
+        NativeFunction.INT32: "int32",
+        NativeFunction.INT64: "int64",
+        NativeFunction.FLOAT32: "float32",
+        NativeFunction.FLOAT64: "float64",
+        NativeFunction.ERF: "erf",
+        NativeFunction.ERFC: "erfc",
+        NativeFunction.ROUND: "round",
+        NativeFunction.ROUND_AWAY_FROM_ZERO: "round_away_from_zero",
     },
 }
 
