@@ -1,16 +1,12 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 from typing import Any, List, Optional, Tuple, Union
 
@@ -36,11 +32,11 @@ class Stmt(common.Stmt):
     pass
 
 
-class Literal(common.Literal, Expr):  # type: ignore
+class Literal(common.Literal, Expr):
     pass
 
 
-class ScalarAccess(common.ScalarAccess, Expr):  # type: ignore
+class ScalarAccess(common.ScalarAccess, Expr):
     pass
 
 
@@ -48,7 +44,7 @@ class VariableKOffset(common.VariableKOffset[Expr]):
     pass
 
 
-class FieldAccess(common.FieldAccess[Expr, VariableKOffset], Expr):  # type: ignore
+class FieldAccess(common.FieldAccess[Expr, VariableKOffset], Expr):
     pass
 
 
@@ -117,7 +113,7 @@ class TernaryOp(common.TernaryOp[Expr], Expr):
     _dtype_propagation = common.ternary_op_dtype_propagation(strict=True)
 
 
-class Cast(common.Cast[Expr], Expr):  # type: ignore
+class Cast(common.Cast[Expr], Expr):
     pass
 
 
@@ -162,22 +158,22 @@ class IJExtent(LocNode):
     j: Tuple[int, int]
 
     @classmethod
-    def zero(cls) -> "IJExtent":
+    def zero(cls) -> IJExtent:
         return cls(i=(0, 0), j=(0, 0))
 
     @classmethod
-    def from_offset(cls, offset: Union[CartesianOffset, VariableKOffset]) -> "IJExtent":
+    def from_offset(cls, offset: Union[CartesianOffset, VariableKOffset]) -> IJExtent:
         if isinstance(offset, VariableKOffset):
             return cls(i=(0, 0), j=(0, 0))
         return cls(i=(offset.i, offset.i), j=(offset.j, offset.j))
 
-    def union(*extents: "IJExtent") -> "IJExtent":
+    def union(*extents: IJExtent) -> IJExtent:
         return IJExtent(
             i=(min(e.i[0] for e in extents), max(e.i[1] for e in extents)),
             j=(min(e.j[0] for e in extents), max(e.j[1] for e in extents)),
         )
 
-    def __add__(self, other: "IJExtent") -> "IJExtent":
+    def __add__(self, other: IJExtent) -> IJExtent:
         return IJExtent(
             i=(self.i[0] + other.i[0], self.i[1] + other.i[1]),
             j=(self.j[0] + other.j[0], self.j[1] + other.j[1]),
@@ -188,17 +184,17 @@ class KExtent(LocNode):
     k: Tuple[int, int]
 
     @classmethod
-    def zero(cls) -> "KExtent":
+    def zero(cls) -> KExtent:
         return cls(k=(0, 0))
 
     @classmethod
-    def from_offset(cls, offset: Union[CartesianOffset, VariableKOffset]) -> "KExtent":
+    def from_offset(cls, offset: Union[CartesianOffset, VariableKOffset]) -> KExtent:
         MAX_OFFSET = 1000
         if isinstance(offset, VariableKOffset):
             return cls(k=(-MAX_OFFSET, MAX_OFFSET))
         return cls(k=(offset.k, offset.k))
 
-    def union(*extents: "KExtent") -> "KExtent":
+    def union(*extents: KExtent) -> KExtent:
         return KExtent(k=(min(e.k[0] for e in extents), max(e.k[1] for e in extents)))
 
 
