@@ -1,25 +1,19 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import warnings
 from typing import List, Set
 
+from gt4py import eve
 from gt4py.cartesian.gtc import gtir
-from gt4py.eve import NodeVisitor
 
 
-class DefinitiveAssignmentAnalysis(NodeVisitor):
+class DefinitiveAssignmentAnalysis(eve.NodeVisitor):
     """
     Analyse gtir stencil expression for access to undefined symbols.
 
@@ -74,6 +68,6 @@ def check(gtir_stencil_expr: gtir.Stencil) -> gtir.Stencil:
     """Execute definitive assignment analysis and warn on errors."""
     invalid_accesses = analyze(gtir_stencil_expr)
     for invalid_access in invalid_accesses:
-        warnings.warn(f"`{invalid_access.name}` may be uninitialized.")
+        warnings.warn(f"`{invalid_access.name}` may be uninitialized.", stacklevel=2)
 
     return gtir_stencil_expr
