@@ -1,26 +1,19 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from typing import List
 
+from gt4py import eve
 from gt4py.cartesian.gtc import oir
-from gt4py.eve import NodeTranslator
-
-from .utils import AccessCollector
+from gt4py.cartesian.gtc.passes.oir_optimizations.utils import AccessCollector
 
 
-class MaskStmtMerging(NodeTranslator):
+class MaskStmtMerging(eve.NodeTranslator):
     def _merge(self, stmts: List[oir.Stmt]) -> List[oir.Stmt]:
         merged = [self.visit(stmts[0])]
         for stmt in stmts[1:]:
@@ -41,9 +34,7 @@ class MaskStmtMerging(NodeTranslator):
 
     def visit_HorizontalExecution(self, node: oir.HorizontalExecution) -> oir.HorizontalExecution:
         return oir.HorizontalExecution(
-            body=self._merge(node.body),
-            declarations=node.declarations,
-            loc=node.loc,
+            body=self._merge(node.body), declarations=node.declarations, loc=node.loc
         )
 
     # Stmt node types with lists of Stmts within them:
