@@ -1,24 +1,21 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import collections
 from typing import Any, Callable, Dict, Set, Union
 
 from gt4py import eve
 from gt4py.cartesian.gtc import oir
-
-from .utils import AccessCollector, collect_symbol_names, symbol_name_creator
+from gt4py.cartesian.gtc.passes.oir_optimizations.utils import (
+    AccessCollector,
+    collect_symbol_names,
+    symbol_name_creator,
+)
 
 
 class TemporariesToScalarsBase(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
@@ -27,9 +24,9 @@ class TemporariesToScalarsBase(eve.NodeTranslator, eve.VisitorWithSymbolTableTra
     ) -> Union[oir.FieldAccess, oir.ScalarAccess]:
         offsets = node.offset.to_dict()
         if node.name in tmps_name_map:
-            assert (
-                offsets["i"] == offsets["j"] == offsets["k"] == 0
-            ), "Non-zero offset in temporary that is replaced?!"
+            assert offsets["i"] == offsets["j"] == offsets["k"] == 0, (
+                "Non-zero offset in temporary that is replaced?!"
+            )
             return oir.ScalarAccess(name=tmps_name_map[node.name], dtype=node.dtype)
         return self.generic_visit(node, tmps_name_map=tmps_name_map, **kwargs)
 
@@ -63,10 +60,7 @@ class TemporariesToScalarsBase(eve.NodeTranslator, eve.VisitorWithSymbolTableTra
         )
 
     def visit_VerticalLoop(
-        self,
-        node: oir.VerticalLoop,
-        tmps_to_replace: Set[str],
-        **kwargs: Any,
+        self, node: oir.VerticalLoop, tmps_to_replace: Set[str], **kwargs: Any
     ) -> oir.VerticalLoop:
         return oir.VerticalLoop(
             loop_order=node.loop_order,
@@ -82,9 +76,7 @@ class TemporariesToScalarsBase(eve.NodeTranslator, eve.VisitorWithSymbolTableTra
             name=node.name,
             params=node.params,
             vertical_loops=self.visit(
-                node.vertical_loops,
-                new_symbol_name=symbol_name_creator(all_names),
-                **kwargs,
+                node.vertical_loops, new_symbol_name=symbol_name_creator(all_names), **kwargs
             ),
             declarations=[d for d in node.declarations if d.name not in tmps_to_replace],
             loc=node.loc,
@@ -101,7 +93,6 @@ class LocalTemporariesToScalars(TemporariesToScalarsBase):
 
     Note that temporaries used in horizontal regions in a single horizontal execution
     may not be scalarized.
-
     """
 
     def visit_Stencil(self, node: oir.Stencil, **kwargs: Any) -> oir.Stencil:
@@ -128,11 +119,10 @@ class LocalTemporariesToScalars(TemporariesToScalarsBase):
 
 
 class WriteBeforeReadTemporariesToScalars(TemporariesToScalarsBase):
-    """Replaces temporay fields that are always written before read by scalars.
+    """Replaces temporary fields that are always written before read by scalars.
 
     Note that temporaries used in horizontal regions in a single horizontal execution
     may not be scalarized.
-
     """
 
     def visit_Stencil(self, node: oir.Stencil, **kwargs: Any) -> oir.Stencil:
