@@ -1,25 +1,19 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import warnings
 from typing import Any
 
+from gt4py import eve
 from gt4py.cartesian.gtc import common, oir
-from gt4py.eve import NodeTranslator
 
 
-class AdjacentLoopMerging(NodeTranslator):
+class AdjacentLoopMerging(eve.NodeTranslator):
     @staticmethod
     def _mergeable(a: oir.VerticalLoop, b: oir.VerticalLoop) -> bool:
         if a.loop_order != b.loop_order:
@@ -38,12 +32,10 @@ class AdjacentLoopMerging(NodeTranslator):
     def _merge(a: oir.VerticalLoop, b: oir.VerticalLoop) -> oir.VerticalLoop:
         sections = a.sections + b.sections
         if a.caches or b.caches:
-            warnings.warn("AdjacentLoopMerging pass removed previously declared caches")
-        return oir.VerticalLoop(
-            loop_order=a.loop_order,
-            sections=sections,
-            caches=[],
-        )
+            warnings.warn(
+                "AdjacentLoopMerging pass removed previously declared caches", stacklevel=2
+            )
+        return oir.VerticalLoop(loop_order=a.loop_order, sections=sections, caches=[])
 
     def visit_Stencil(self, node: oir.Stencil, **kwargs: Any) -> oir.Stencil:
         if not node.vertical_loops:
