@@ -1,20 +1,15 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from abc import abstractmethod
 from typing import Callable, Optional, Protocol, Sequence, Type, Union
 
+from gt4py import eve
 from gt4py.cartesian.gtc import oir
 from gt4py.cartesian.gtc.passes.oir_optimizations.caches import (
     IJCacheDetection,
@@ -37,10 +32,9 @@ from gt4py.cartesian.gtc.passes.oir_optimizations.temporaries import (
     WriteBeforeReadTemporariesToScalars,
 )
 from gt4py.cartesian.gtc.passes.oir_optimizations.vertical_loop_merging import AdjacentLoopMerging
-from gt4py.eve.visitors import NodeVisitor
 
 
-PassT = Union[Callable[[oir.Stencil], oir.Stencil], Type[NodeVisitor]]
+PassT = Union[Callable[[oir.Stencil], oir.Stencil], Type[eve.NodeVisitor]]
 
 
 class OirPipeline(Protocol):
@@ -103,8 +97,8 @@ class DefaultPipeline(OirPipeline):
 
     def run(self, oir: oir.Stencil) -> oir.Stencil:
         for step in self.steps:
-            if isinstance(step, type) and issubclass(step, NodeVisitor):
+            if isinstance(step, type) and issubclass(step, eve.NodeVisitor):
                 oir = step().visit(oir)
             else:
-                oir = step(oir)
+                oir = step(oir)  # type: ignore[call-arg, assignment]
         return oir
