@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import collections
 import inspect
@@ -18,7 +12,7 @@ import types
 
 import numpy as np
 
-import gt4py.cartesian.utils as gt_utils
+from gt4py.cartesian import utils as gt_utils
 
 
 def copy_func(f, name=None):
@@ -44,9 +38,9 @@ def standardize_dtype_dict(dtypes):
     dtypes as 1-tuples)
     """
     assert isinstance(dtypes, collections.abc.Mapping)
-    assert all(
-        (isinstance(k, str) or gt_utils.is_iterable_of(k, str)) for k in dtypes.keys()
-    ), "Invalid key in 'dtypes'."
+    assert all((isinstance(k, str) or gt_utils.is_iterable_of(k, str)) for k in dtypes.keys()), (
+        "Invalid key in 'dtypes'."
+    )
     assert all(
         (
             isinstance(k, (type, np.dtype))
