@@ -1,20 +1,13 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Tools for source code generation."""
 
-
 from __future__ import annotations
 
 import abc
@@ -31,7 +24,7 @@ import types
 
 import black
 import jinja2
-from mako import template as mako_tpl  # type: ignore[import]
+from mako import template as mako_tpl
 
 from . import exceptions, utils
 from .concepts import CollectionNode, LeafNode, Node, RootNode
@@ -112,10 +105,7 @@ def format_python_source(
 ) -> str:
     """Format Python source code using black formatter."""
     python_versions = python_versions or {f"{sys.version_info.major}{sys.version_info.minor}"}
-    target_versions = set(
-        black.TargetVersion[f"PY{v.replace('.', '')}"]  # type: ignore[attr-defined]
-        for v in python_versions
-    )
+    target_versions = set(black.TargetVersion[f"PY{v.replace('.', '')}"] for v in python_versions)  # type: ignore[attr-defined]  # .TargetVersion implicitly exported
 
     formatted_source = black.format_str(
         source,
@@ -158,7 +148,7 @@ if _CLANG_FORMAT_EXECUTABLE is not None:
     ) -> str:
         """Format C++ source code using clang-format."""
         assert isinstance(_CLANG_FORMAT_EXECUTABLE, str)
-        args = [_CLANG_FORMAT_EXECUTABLE]
+        args = [_CLANG_FORMAT_EXECUTABLE, "--assume-filename=_gt4py_generated_file.cpp"]
         if style:
             args.append(f"--style={style}")
         if fallback_style:
@@ -308,13 +298,13 @@ class TextBlock:
         common `indent - append - dedent` workflows.
 
         Examples:
-            >>> block = TextBlock();
-            >>> block.append('first line')  # doctest: +ELLIPSIS
+            >>> block = TextBlock()
+            >>> block.append("first line")  # doctest: +ELLIPSIS
             <...>
             >>> with block.indented():
-            ...     block.append('second line');  # doctest: +ELLIPSIS
+            ...     block.append("second line")  # doctest: +ELLIPSIS
             <...>
-            >>> block.append('third line')  # doctest: +ELLIPSIS
+            >>> block.append("third line")  # doctest: +ELLIPSIS
             <...>
             >>> print(block.text)
             first line
@@ -357,7 +347,7 @@ TemplateT = TypeVar("TemplateT", bound="Template")
 class Template(Protocol):
     """Protocol (abstract base class) defining the Template interface.
 
-    Direct subclassess of this base class only need to implement the
+    Direct subclasses of this base class only need to implement the
     abstract methods to adapt different template engines to this
     interface.
 
@@ -479,7 +469,9 @@ class StringTemplate(BaseTemplate):
                 message += f" (created at {self.definition_loc[0]}:{self.definition_loc[1]})"
             try:
                 loc_info = re.search(r"line (\d+), col (\d+)", str(e))
-                message += f" rendering error at template line: {loc_info[1]}, column {loc_info[2]}."  # type: ignore
+                message += (
+                    f" rendering error at template line: {loc_info[1]}, column {loc_info[2]}."  # type: ignore
+                )
             except Exception:
                 message += " rendering error."
 
@@ -544,7 +536,9 @@ class MakoTemplate(BaseTemplate):
             if self.definition_loc:
                 message += f" created at {self.definition_loc[0]}:{self.definition_loc[1]}"
                 try:
-                    message += f" (error likely around line {e.lineno}, column: {getattr(e, 'pos', '?')})"  # type: ignore  # assume Mako exception
+                    message += (
+                        f" (error likely around line {e.lineno}, column: {getattr(e, 'pos', '?')})"  # type: ignore  # assume Mako exception
+                    )
                 except Exception:
                     message = f"{message}:\n---\n{definition}\n---\n"
 
@@ -644,26 +638,24 @@ class TemplatedGenerator(NodeVisitor):
 
     @overload
     @classmethod
-    def apply(cls, root: LeafNode, **kwargs: Any) -> str:
-        ...
+    def apply(cls, root: LeafNode, **kwargs: Any) -> str: ...
 
     @overload
     @classmethod
-    def apply(  # noqa: F811  # redefinition of symbol
+    def apply(  # redefinition of symbol
         cls, root: CollectionNode, **kwargs: Any
-    ) -> Collection[str]:
-        ...
+    ) -> Collection[str]: ...
 
     @classmethod
-    def apply(  # noqa: F811  # redefinition of symbol
+    def apply(  # redefinition of symbol
         cls, root: RootNode, **kwargs: Any
     ) -> Union[str, Collection[str]]:
         """Public method to build a class instance and visit an IR node.
 
         Args:
             root: An IR node.
-            node_templates (optiona): see :class:`NodeDumper`.
-            dump_function (optiona): see :class:`NodeDumper`.
+            node_templates (optional): see :class:`NodeDumper`.
+            dump_function (optional): see :class:`NodeDumper`.
             ``**kwargs`` (optional): custom extra parameters forwarded to `visit_NODE_TYPE_NAME()`.
 
         Returns:
@@ -680,6 +672,23 @@ class TemplatedGenerator(NodeVisitor):
         """
         return str(node)
 
+    @overload
+    def generic_visit(self, node: Node, **kwargs: Any) -> str: ...
+
+    @overload
+    def generic_visit(
+        self,
+        node: Union[
+            list,
+            tuple,
+            collections.abc.Set,
+            collections.abc.Sequence,
+            dict,
+            collections.abc.Mapping,
+        ],
+        **kwargs: Any,
+    ) -> Collection[str]: ...
+
     def generic_visit(self, node: RootNode, **kwargs: Any) -> Union[str, Collection[str]]:
         if isinstance(node, Node):
             template, key = self.get_template(node)
