@@ -117,7 +117,23 @@ class NdArrayField(
     _domain: common.Domain
     _ndarray: core_defs.NDArrayObject
 
-    array_ns: ClassVar[ModuleType]  # TODO(havogt) introduce a NDArrayNamespace protocol
+    array_ns: ClassVar[ModuleType]  # TODO(havogt): introduce a NDArrayNamespace protocol
+    array_byte_bounds: ClassVar[  # TODO(egparedes): make this part of the previous protocol
+        Callable[[npt.NDArray], tuple[int, int]]
+        | Callable[[core_defs.NDArrayObject], tuple[int, int]]
+    ]
+
+    def __init_subclass__(cls, **kwargs: Any) -> None:
+        super().__init_subclass__(**kwargs)
+        if hasattr(cls, "array_ns") and not hasattr(cls, "array_byte_bounds"):
+            # This is needed to initialize `array_byte_bounds` only once per subclass
+            try:
+                cls.array_byte_bounds = staticmethod(
+                    getattr(cls.array_ns, "byte_bounds", None)
+                    or cls.array_ns.lib.array_utils.byte_bounds
+                )
+            except AttributeError:
+                pass
 
     @property
     def domain(self) -> common.Domain:
@@ -127,7 +143,7 @@ class NdArrayField(
     def shape(self) -> tuple[int, ...]:
         return self._ndarray.shape
 
-    @property
+    @functools.cached_property
     def __gt_origin__(self) -> tuple[int, ...]:
         assert common.Domain.is_finite(self._domain)
         return tuple(-r.start for r in self._domain.ranges)
@@ -154,7 +170,7 @@ class NdArrayField(
     def codomain(self) -> type[core_defs.ScalarT]:
         return self.dtype.scalar_type
 
-    @property
+    @functools.cached_property
     def dtype(self) -> core_defs.DType[core_defs.ScalarT]:
         return core_defs.dtype(self._ndarray.dtype.type)
 
@@ -428,19 +444,28 @@ class NdArrayField(
         assert common.is_relative_index_sequence(slice_)
         return new_domain, slice_
 
+    @property
+    def _data_buffer_ptr_(self) -> int:
+        """
+        Returns the pointer of the underlying data buffer.
+
+        Subclasses should redefine this property with the fastest possible
+        implementation.
+
+        This is mainly needed to interface with high-performance code generated by
+        gt4py backends. Therefore, this is considered for now an internal attribute
+        and it should not be accessed directly by users.
+        """
+        return self.array_byte_bounds(self._ndarray)[0]  # type: ignore[call-arg,misc] # array_byte_bounds is a staticmethod
+
     if dace:
-        # Extension of NdArrayField adding SDFGConvertible support in GT4Py Programs
+
         def _dace_data_ptr(self) -> int:
-            array_ns = self.array_ns
-            array_byte_bounds = (  # TODO(egparedes): make this part of some Array namespace protocol
-                array_ns.byte_bounds
-                if hasattr(array_ns, "byte_bounds")
-                else array_ns.lib.array_utils.byte_bounds
-            )
-            return array_byte_bounds(self.ndarray)[0]
+            return self._data_buffer_ptr_
 
         def _dace_descriptor(self) -> dace.data.Data:
             return dace.data.create_datadescriptor(self.ndarray)
+
     else:
 
         def _dace_data_ptr(self) -> int:
@@ -448,17 +473,28 @@ class NdArrayField(
                 "data_ptr is only supported when the 'dace' module is available."
             )
 
-        def _dace_descriptor(self) -> Any:
+        def _dace_descriptor(self) -> dace.data.Data:
             raise NotImplementedError(
                 "__descriptor__ is only supported when the 'dace' module is available."
             )
 
     data_ptr = _dace_data_ptr
+    """
+    Returns the pointer of the underlying data buffer.
+
+    Fully equivalent to `self._data_buffer_ptr_`. It is only defined to emulate the
+    PyTorch API for DaCe interoperability.
+
+    Note:
+        This method is experimental and will be likely removed in future versions.
+    """
+
     __descriptor__ = _dace_descriptor
+    """Extension of NdArrayField adding SDFGConvertible support in GT4Py Programs."""
 
 
 @dataclasses.dataclass(frozen=True)
-class NdArrayConnectivityField(  # type: ignore[misc] # for __ne__, __eq__
+class NdArrayConnectivityField(
     common.Connectivity[common.DimsT, common.DimT],
     NdArrayField[common.DimsT, core_defs.IntegralScalar],
 ):
@@ -480,8 +516,7 @@ class NdArrayConnectivityField(  # type: ignore[misc] # for __ne__, __eq__
         raise NotImplementedError()
 
     @property
-    # type: ignore[override] # TODO(havogt): instead of inheriting from NdArrayField, steal implementation or common base
-    def codomain(self) -> common.DimT:
+    def codomain(self) -> common.DimT:  # type: ignore[override] # TODO(havogt): instead of inheriting from NdArrayField, steal implementation or common base
         return self._codomain
 
     @property
@@ -996,9 +1031,9 @@ def _make_reduction(
         reduce_dim_index = field.domain.dims.index(axis)
         current_offset_provider = embedded_context.get_offset_provider(None)
         assert current_offset_provider is not None
-        offset_definition = current_offset_provider[
-            axis.value
-        ]  # assumes offset and local dimension have same name
+        offset_definition = common.get_offset(
+            current_offset_provider, axis.value
+        )  # assumes offset and local dimension have same name
         assert common.is_neighbor_table(offset_definition)
         new_domain = common.Domain(*[nr for nr in field.domain if nr.dim != axis])
 
@@ -1040,6 +1075,13 @@ _nd_array_implementations = [np]
 class NumPyArrayField(NdArrayField):
     array_ns: ClassVar[ModuleType] = np
 
+    # It is only possible to cache the data buffer pointer in this way
+    # because the backing np.ndarray is never replaced after creation,
+    # since this is a frozen dataclass.
+    @functools.cached_property
+    def _data_buffer_ptr_(self) -> int:
+        return self._ndarray.ctypes.data  # type: ignore[attr-defined]  # np.ndarray has `ctypes` attribute
+
 
 common._field.register(np.ndarray, NumPyArrayField.from_array)
 
@@ -1048,6 +1090,8 @@ common._field.register(np.ndarray, NumPyArrayField.from_array)
 class NumPyArrayConnectivityField(NdArrayConnectivityField):
     array_ns: ClassVar[ModuleType] = np
 
+    _data_buffer_ptr_ = functools.cached_property(NumPyArrayField._data_buffer_ptr_.func)  # type: ignore[attr-defined]  # cached_property has `func` attribute
+
 
 common._connectivity.register(np.ndarray, NumPyArrayConnectivityField.from_array)
 
@@ -1055,16 +1099,26 @@ common._connectivity.register(np.ndarray, NumPyArrayConnectivityField.from_array
 if cp:
     _nd_array_implementations.append(cp)
 
+    # Same as in the NumPy case:
+    # It is only possible to cache the data buffer pointer in this way
+    # because the backing np.ndarray is never replaced after creation
+    # since this is a frozen dataclass.
     @dataclasses.dataclass(frozen=True, eq=False)
     class CuPyArrayField(NdArrayField):
         array_ns: ClassVar[ModuleType] = cp
 
+        @functools.cached_property
+        def _data_buffer_ptr_(self) -> int:
+            return self._ndarray.data.ptr  # type: ignore[attr-defined]  # cp.ndarray has `data` attribute
+
     common._field.register(cp.ndarray, CuPyArrayField.from_array)
 
     @dataclasses.dataclass(frozen=True, eq=False)
     class CuPyArrayConnectivityField(NdArrayConnectivityField):
         array_ns: ClassVar[ModuleType] = cp
 
+        _data_buffer_ptr_ = functools.cached_property(CuPyArrayField._data_buffer_ptr_.func)  # type: ignore[attr-defined]  # cached_property has `func` attribute
+
     common._connectivity.register(cp.ndarray, CuPyArrayConnectivityField.from_array)
 
 # JAX
