@@ -8,7 +8,7 @@
 
 """Contains the exception classes and other utilities for error handling."""
 
-from . import excepthook  # noqa: F401 [unused-import]
+from . import excepthook  # Necessary for replacing the sys.excepthook # noqa: F401 [unused-import]
 from .exceptions import (
     CompilationError,
     DSLError,
