@@ -6,7 +6,7 @@
 # Please, refer to the LICENSE file in the root directory.
 # SPDX-License-Identifier: BSD-3-Clause
 
-from gt4py.next.ffront import field_operator_ast as foast
+from gt4py.next.ffront import dialect_ast_enums, field_operator_ast as foast
 
 
 def compute_assign_indices(
@@ -52,3 +52,31 @@ def compute_assign_indices(
             *list(range(starred_upper, num_elts)),
         ]
     return list(range(0, num_elts))  # no starred target
+
+
+def expr_to_index(expr: foast.Expr) -> int:
+    """
+    Convert an expression that represent an (integral) index to an int index.
+
+    Args:
+        expr: The expression to convert. Supported are literals of the form `+/-<int>`.
+
+    Returns:
+        The integer value of the index.
+
+    Raises:
+        ValueError: If the expression is not a valid index expression.
+    """
+    if isinstance(expr, foast.Constant):
+        return expr.value
+    if (
+        isinstance(expr, foast.UnaryOp)
+        and isinstance(expr.op, dialect_ast_enums.UnaryOperator)
+        and isinstance(expr.operand, foast.Constant)
+    ):
+        if expr.op is dialect_ast_enums.UnaryOperator.USUB:
+            return -expr.operand.value
+        if expr.op is dialect_ast_enums.UnaryOperator.UADD:
+            return expr.operand.value
+
+    raise ValueError(f"Not an index: '{expr}'.")
