@@ -412,6 +412,11 @@ def concat_where(*args):
     raise BackendNotSelectedError()
 
 
+@builtin_dispatch
+def get_domain_range(*args):
+    raise BackendNotSelectedError()
+
+
 UNARY_MATH_NUMBER_BUILTINS = {"abs", "neg"}
 UNARY_LOGICAL_BUILTINS = {"not_"}
 UNARY_MATH_FP_BUILTINS = {
@@ -485,6 +490,7 @@ BUILTINS = {
     "cartesian_domain",
     "cast_",
     "deref",
+    "get_domain_range",
     "if_",
     "index",  # `index(dim)` creates a dim-field that has the current index at each point
     "shift",
