@@ -7,7 +7,7 @@
 # SPDX-License-Identifier: BSD-3-Clause
 
 import typing
-from typing import Any, Callable, Optional, Union
+from typing import Any, Callable, Iterable, Optional, TypeAlias, Union
 
 from gt4py._core import definitions as core_defs
 from gt4py.next import common
@@ -15,6 +15,9 @@ from gt4py.next.iterator import builtins, ir as itir
 from gt4py.next.type_system import type_specifications as ts, type_translation
 
 
+ExprLike: TypeAlias = Union[str, core_defs.Scalar, common.Dimension, itir.Expr]
+
+
 def sym(sym_or_name: Union[str, itir.Sym], type_: str | ts.TypeSpec | None = None) -> itir.Sym:
     """
     Convert to Sym if necessary.
@@ -68,7 +71,7 @@ def ref(
     return ref
 
 
-def ensure_expr(literal_or_expr: Union[str, core_defs.Scalar, itir.Expr]) -> itir.Expr:
+def ensure_expr(expr_like: ExprLike) -> itir.Expr:
     """
     Convert literals into a SymRef or Literal and let expressions pass unchanged.
 
@@ -83,14 +86,16 @@ def ensure_expr(literal_or_expr: Union[str, core_defs.Scalar, itir.Expr]) -> iti
     >>> ensure_expr(itir.OffsetLiteral(value="i"))
     OffsetLiteral(value='i')
     """
-    if isinstance(literal_or_expr, str):
-        return ref(literal_or_expr)
-    elif core_defs.is_scalar_type(literal_or_expr):
-        return literal_from_value(literal_or_expr)
-    elif literal_or_expr is None:
+    if isinstance(expr_like, str):
+        return ref(expr_like)
+    elif core_defs.is_scalar_type(expr_like):
+        return literal_from_value(expr_like)
+    elif expr_like is None:
         return itir.NoneLiteral()
-    assert isinstance(literal_or_expr, itir.Expr), literal_or_expr
-    return literal_or_expr
+    elif isinstance(expr_like, common.Dimension):
+        return axis_literal(expr_like)
+    assert isinstance(expr_like, itir.Expr), expr_like
+    return expr_like
 
 
 def ensure_offset(str_or_offset: Union[str, int, itir.OffsetLiteral]) -> itir.OffsetLiteral:
@@ -307,14 +312,17 @@ def shift(offset, value=None):
     if value is not None:
         if isinstance(value, int):
             value = ensure_offset(value)
-        elif isinstance(value, str):
-            value = ref(value)
+        if isinstance(value, itir.Literal) and value.type.kind in (
+            ts.ScalarKind.INT32,
+            ts.ScalarKind.INT64,
+        ):
+            value = itir.OffsetLiteral(value=int(value.value))
         args.append(value)
     return call(call("shift")(*args))
 
 
-def literal(value: str, typename: str) -> itir.Literal:
-    return itir.Literal(value=value, type=ensure_type(typename))
+def literal(value: str, type_: str | ts.TypeSpec) -> itir.Literal:
+    return itir.Literal(value=value, type=ensure_type(type_))
 
 
 def literal_from_value(val: core_defs.Scalar) -> itir.Literal:
@@ -378,7 +386,7 @@ def lifted_neighbors(offset, it) -> itir.Expr:
 
 
 def as_fieldop_neighbors(
-    offset: str | itir.OffsetLiteral, it: str | itir.Expr, domain: Optional[itir.FunCall] = None
+    offset: str | itir.OffsetLiteral, field: str | itir.Expr, domain: Optional[itir.FunCall] = None
 ) -> itir.Expr:
     """
     Create a fieldop for neighbors call.
@@ -388,7 +396,21 @@ def as_fieldop_neighbors(
     >>> str(as_fieldop_neighbors("off", "a"))
     '(⇑(λ(it) → neighbors(offₒ, it)))(a)'
     """
-    return as_fieldop(lambda_("it")(neighbors(offset, "it")), domain)(it)
+    return as_fieldop(lambda_("it")(neighbors(offset, "it")), domain)(field)
+
+
+def as_fieldop_deref_list_get(
+    list_idx: str | itir.Expr, local_field: str | itir.Expr, domain: Optional[itir.FunCall] = None
+) -> itir.Expr:
+    """
+    Create a fieldop for list_get call.
+
+    Examples
+    --------
+    >>> str(as_fieldop_deref_list_get("idx", "lst"))
+    '(⇑(λ(it) → list_get(idx, ·it)))(lst)'
+    """
+    return as_fieldop(lambda_("it")(list_get(list_idx, deref("it"))), domain)(local_field)
 
 
 def promote_to_const_iterator(expr: str | itir.Expr) -> itir.Expr:
@@ -445,8 +467,8 @@ def domain(
         grid_type = f"{grid_type!s}_domain"
     expr = call(grid_type)(
         *[
-            call("named_range")(
-                axis_literal(d),
+            named_range(
+                d,
                 r[0],
                 r[1],
             )
@@ -457,6 +479,36 @@ def domain(
     return expr
 
 
+def get_field_domain(
+    grid_type: Union[common.GridType, str],
+    field: str | itir.Expr,
+    dims: Iterable[common.Dimension] | None = None,
+) -> itir.Expr:
+    if isinstance(field, itir.Expr) and isinstance(field.type, ts.FieldType):
+        assert dims is None or all(d1 == d2 for d1, d2 in zip(field.type.dims, dims, strict=True))
+        dims = field.type.dims
+    else:
+        if dims is None:
+            raise ValueError("Field expression must be typed if 'dims' is not given.")
+
+    return domain(
+        grid_type,
+        {
+            dim: (
+                tuple_get(0, call("get_domain_range")(field, dim)),
+                tuple_get(1, call("get_domain_range")(field, dim)),
+            )
+            for dim in dims
+        },
+    )
+
+
+def named_range(
+    dim: itir.AxisLiteral | common.Dimension, start: itir.Expr, stop: itir.Expr
+) -> itir.FunCall:
+    return call("named_range")(dim, start, stop)
+
+
 def as_fieldop(expr: itir.Expr | str, domain: Optional[itir.Expr] = None) -> Callable:
     """
     Create an `as_fieldop` call.
