@@ -132,14 +132,17 @@ def _collect_offset_definitions(
     grid_type: common.GridType,
     offset_provider_type: common.OffsetProviderType,
 ) -> dict[str, TagDefinition]:
-    used_offset_tags: set[itir.OffsetLiteral] = (
+    used_offset_tags: set[str] = (
         node.walk_values()
         .if_isinstance(itir.OffsetLiteral)
         .filter(lambda offset_literal: isinstance(offset_literal.value, str))
         .getattr("value")
     ).to_set()
-    if not used_offset_tags.issubset(set(offset_provider_type.keys())):
-        raise AssertionError("ITIR contains an offset tag without a corresponding offset provider.")
+    # implicit offsets don't occur in the `offset_provider_type`, get them from the used offset tags
+    offset_provider_type = {
+        offset_name: common.get_offset_type(offset_provider_type, offset_name)
+        for offset_name in used_offset_tags
+    } | {**offset_provider_type}
     offset_definitions = {}
 
     for offset_name, dim_or_connectivity_type in offset_provider_type.items():
@@ -153,17 +156,6 @@ def _collect_offset_definitions(
                 )
             else:
                 assert grid_type == common.GridType.UNSTRUCTURED
-                # TODO(tehrengruber): The implicit offset providers added to support syntax like
-                #  `KDim+1` can also include horizontal dimensions. Cartesian shifts in this
-                #  dimension are not supported by the backend and also never occur in user code.
-                #  We just skip these here for now, but this is not a clean solution. Not having
-                #  any unstructured dimensions in here would be preferred.
-                if (
-                    dim.kind == common.DimensionKind.HORIZONTAL
-                    and offset_name == common.dimension_to_implicit_offset(dim.value)
-                ):
-                    continue
-
                 if not dim.kind == common.DimensionKind.VERTICAL:
                     raise ValueError(
                         "Mapping an offset to a horizontal dimension in unstructured is not allowed."
@@ -464,13 +456,22 @@ class GTFN_lowering(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
             shift_offsets = self._collect_offset_or_axis_node(itir.OffsetLiteral, kwargs["stencil"])
             for o in shift_offsets:
                 if o in self.offset_provider_type and isinstance(
-                    self.offset_provider_type[o], common.NeighborConnectivityType
+                    common.get_offset_type(self.offset_provider_type, o),
+                    common.NeighborConnectivityType,
                 ):
                     connectivities.append(SymRef(id=o))
         return UnstructuredDomain(
             tagged_sizes=sizes, tagged_offsets=domain_offsets, connectivities=connectivities
         )
 
+    def _visit_get_domain_range(self, node: itir.FunCall, **kwargs: Any) -> Node:
+        field, dim = node.args
+
+        return FunCall(
+            fun=SymRef(id="get_domain_range"),
+            args=[self.visit(field, **kwargs), self.visit(dim, **kwargs)],
+        )
+
     def visit_FunCall(self, node: itir.FunCall, **kwargs: Any) -> Node:
         if isinstance(node.fun, itir.SymRef):
             if node.fun.id in self._unary_op_map:
