@@ -42,10 +42,13 @@ from gt4py.next.program_processors.runners.dace import (
 from gt4py.next.type_system import type_info as ti, type_specifications as ts
 
 
-# Magic local dimension for the result of a `make_const_list`.
-# A clean implementation will probably involve to tag the `make_const_list`
-# with the neighborhood it is meant to be used with.
-_CONST_DIM = gtx_common.Dimension(value="_CONST_DIM", kind=gtx_common.DimensionKind.LOCAL)
+# Magic local dimension used for list of values with length known at compile-time.
+_CONST_DIM: Final = gtx_common.Dimension(value="_CONST_DIM", kind=gtx_common.DimensionKind.LOCAL)
+
+# Data type of the index values in connectivity tables.
+_INDEX_DTYPE: Final = ts.ScalarType(
+    kind=getattr(ts.ScalarKind, gtir_builtins.INTEGER_INDEX_BUILTIN.upper())
+)
 
 
 @dataclasses.dataclass(frozen=True)
@@ -67,22 +70,37 @@ class ValueExpr:
     dc_node: dace.nodes.AccessNode
     gt_dtype: ts.ListType | ts.ScalarType
 
+    def __post_init__(self) -> None:
+        if isinstance(self.gt_dtype, ts.ListType):
+            assert self.gt_dtype.offset_type is not None
+            assert self.gt_dtype.offset_type.kind == gtx_common.DimensionKind.LOCAL
+
 
 @dataclasses.dataclass(frozen=True)
 class MemletExpr:
     """
-    Scalar or array data access through a memlet.
+    Memlet to retrieve local data, either a scalar or a list, from a field.
 
     Args:
-        dc_node: Access node to the data container, can be either a scalar or a local list.
-        gt_dtype: GT4Py data type, which includes the `offset_type` local dimension for lists.
-        subset: Represents the subset to use in memlet to access the above data.
+        dc_node: Access node to the array data container of the source field.
+        gt_field: GT4Py field type definition, which includes the list of dimensions
+          according to the canonical field layout, used to modify the memlet subset.
+        subset: The memlet subset to retrieve the local data.
     """
 
     dc_node: dace.nodes.AccessNode
-    gt_dtype: ts.ListType | ts.ScalarType
+    gt_field: ts.FieldType
     subset: dace_subsets.Range
 
+    @property
+    def gt_dtype(self) -> ts.ScalarType | ts.ListType:
+        return self.gt_field.dtype
+
+    def __post_init__(self) -> None:
+        if isinstance(self.gt_dtype, ts.ListType):
+            assert self.gt_dtype.offset_type is not None
+            assert self.gt_dtype.offset_type.kind == gtx_common.DimensionKind.LOCAL
+
 
 @dataclasses.dataclass(frozen=True)
 class SymbolExpr:
@@ -117,8 +135,15 @@ class IteratorExpr:
     field_domain: list[tuple[gtx_common.Dimension, dace.symbolic.SymbolicType]]
     indices: dict[gtx_common.Dimension, DataExpr]
 
+    def __post_init__(self) -> None:
+        gtx_common.check_dims([dim for dim, _ in self.field_domain])
+        if isinstance(self.gt_dtype, ts.ListType):
+            assert self.gt_dtype.offset_type is not None
+            assert self.gt_dtype.offset_type.kind == gtx_common.DimensionKind.LOCAL
+            assert all(dim != self.gt_dtype.offset_type for dim, _ in self.field_domain)
+
     def get_field_type(self) -> ts.FieldType:
-        return ts.FieldType([dim for dim, _ in self.field_domain], self.gt_dtype)
+        return ts.FieldType(dims=[dim for dim, _ in self.field_domain], dtype=self.gt_dtype)
 
     def get_memlet_subset(self, sdfg: dace.SDFG) -> dace_subsets.Range:
         if len(self.field_domain) == 0:  # zero-dimensional field
@@ -130,8 +155,17 @@ class IteratorExpr:
         field_desc = self.field.desc(sdfg)
         if isinstance(self.gt_dtype, ts.ListType):
             assert len(field_desc.shape) == len(self.field_domain) + 1
-            assert self.gt_dtype.offset_type is not None
-            field_domain = [*self.field_domain, (self.gt_dtype.offset_type, 0)]
+            local_dim: gtx_common.Dimension = self.gt_dtype.offset_type  # type: ignore[assignment]  # checked in __post_init__
+            # construct the full subset according to the canonical field domain
+            sorted_dims = gtx_common.order_dimensions(
+                [dim for dim, _ in self.field_domain] + [local_dim]
+            )
+            local_dim_index = sorted_dims.index(local_dim)
+            field_domain = [
+                *self.field_domain[:local_dim_index],
+                (local_dim, 0),
+                *self.field_domain[local_dim_index:],
+            ]
         else:
             assert len(field_desc.shape) == len(self.field_domain)
             field_domain = self.field_domain
@@ -233,17 +267,28 @@ class DataflowOutputEdge:
         self,
         map_exit: Optional[dace.nodes.MapExit],
         dest: dace.nodes.AccessNode,
-        subset: dace_subsets.Range,
-    ) -> None:
+        dest_subset: dace_subsets.Range,
+    ) -> bool:
+        """Create the connection.
+
+        Might remove the last data container. Returns `True` if it was removed,
+        `False` if kept.
+        """
         write_edge = self.state.in_edges(self.result.dc_node)[0]
-        write_size = write_edge.data.dst_subset.num_elements()
+        write_size = (
+            dace.symbolic.SymExpr(1)  # subset `None` not expected, but it can appear for scalars
+            if write_edge.data.dst_subset is None
+            else write_edge.data.dst_subset.num_elements()
+        )
         # check the kind of node which writes the result
         if isinstance(write_edge.src, dace.nodes.Tasklet):
             # The temporary data written by a tasklet can be safely deleted
-            assert write_size.is_constant()
+            assert isinstance(write_size, int) or str(write_size).isdigit()
             remove_last_node = True
         elif isinstance(write_edge.src, dace.nodes.NestedSDFG):
-            if write_size.is_constant():
+            # TODO(phimuell, edopao): We need a better justification here, best would
+            #   be a reference to a DaCe/GT4Py issue on GH.
+            if isinstance(write_size, int) or str(write_size).isdigit():
                 # Temporary data with compile-time size is allocated on the stack
                 # and therefore is safe to keep. We decide to keep it as a workaround
                 # for a dace issue with memlet propagation in combination with
@@ -263,30 +308,34 @@ class DataflowOutputEdge:
             remove_last_node = False
 
         if remove_last_node:
-            last_node = write_edge.src
-            last_node_connector = write_edge.src_conn
+            src_node = write_edge.src
+            src_node_connector = write_edge.src_conn
+            src_subset = write_edge.data.src_subset
             self.state.remove_node(self.result.dc_node)
         else:
-            last_node = self.result.dc_node
-            last_node_connector = None
+            src_node = write_edge.dst
+            src_node_connector = None
+            src_subset = write_edge.data.dst_subset
 
         if map_exit is None:
             self.state.add_edge(
-                last_node,
-                last_node_connector,
+                src_node,
+                src_node_connector,
                 dest,
                 None,
-                dace.Memlet(data=dest.data, subset=subset),
+                dace.Memlet(data=dest.data, subset=dest_subset, other_subset=src_subset),
             )
         else:
             self.state.add_memlet_path(
-                last_node,
+                src_node,
                 map_exit,
                 dest,
-                src_conn=last_node_connector,
-                memlet=dace.Memlet(data=dest.data, subset=subset),
+                src_conn=src_node_connector,
+                memlet=dace.Memlet(data=dest.data, subset=dest_subset, other_subset=src_subset),
             )
 
+        return remove_last_node
+
 
 DACE_REDUCTION_MAPPING: dict[str, dace.dtypes.ReductionType] = {
     "minimum": dace.dtypes.ReductionType.Min,
@@ -373,17 +422,8 @@ class LambdaToDataflow(eve.NodeVisitor):
         src_subset: dace_subsets.Range,
         dst_node: dace.nodes.Node,
         dst_conn: Optional[str] = None,
-        src_offset: Optional[list[dace.symbolic.SymExpr]] = None,
     ) -> None:
-        input_subset = (
-            src_subset
-            if src_offset is None
-            else dace_subsets.Range(
-                (start - off, stop - off, step)
-                for (start, stop, step), off in zip(src_subset, src_offset, strict=True)
-            )
-        )
-        edge = MemletInputEdge(self.state, src, input_subset, dst_node, dst_conn)
+        edge = MemletInputEdge(self.state, src, src_subset, dst_node, dst_conn)
         self.input_edges.append(edge)
 
     def _add_edge(
@@ -484,7 +524,7 @@ class LambdaToDataflow(eve.NodeVisitor):
             local_view_node = self.state.add_access(view)
             self._add_input_data_edge(field.dc_node, field.subset, local_view_node)
 
-            return ValueExpr(local_view_node, desc.dtype)
+            return ValueExpr(local_view_node, field.gt_dtype)
 
         else:
             return field
@@ -555,7 +595,11 @@ class LambdaToDataflow(eve.NodeVisitor):
             # deref a zero-dimensional field
             assert len(arg_expr.field_domain) == 0
             assert isinstance(node.type, ts.ScalarType)
-            return MemletExpr(arg_expr.field, arg_expr.gt_dtype, subset="0")
+            return MemletExpr(
+                dc_node=arg_expr.field,
+                gt_field=ts.FieldType(dims=[], dtype=arg_expr.gt_dtype),
+                subset="0",
+            )
 
         # handle default case below: deref a field with one or more dimensions
 
@@ -564,7 +608,7 @@ class LambdaToDataflow(eve.NodeVisitor):
         if all(isinstance(index, SymbolExpr) for index in arg_expr.indices.values()):
             # when all indices are symbolic expressions, we can perform direct field access through a memlet
             field_subset = arg_expr.get_memlet_subset(self.sdfg)
-            return MemletExpr(arg_expr.field, arg_expr.gt_dtype, field_subset)
+            return MemletExpr(arg_expr.field, arg_expr.get_field_type(), field_subset)
 
         # when any of the indices is a runtime value (either a dynamic cartesian
         # offset or a connectivity offset), the deref is lowered to a tasklet
@@ -580,10 +624,10 @@ class LambdaToDataflow(eve.NodeVisitor):
         # an index can be either a connector name (for dynamic/indirect indices)
         # or a symbol value (for literal values and scalar arguments).
         index_internals = ",".join(
-            str(index.value)
-            if isinstance(index, SymbolExpr)
-            else IndexConnectorFmt.format(dim=dim.value)
-            for dim, index in field_indices
+            str(index.value - offset)
+            if isinstance(index := arg_expr.indices[dim], SymbolExpr)
+            else f"{IndexConnectorFmt.format(dim=dim.value)} - {offset}"
+            for (dim, offset) in arg_expr.field_domain
         )
         deref_node = self._add_tasklet(
             "deref",
@@ -597,7 +641,6 @@ class LambdaToDataflow(eve.NodeVisitor):
             dace_subsets.Range.from_array(field_desc),
             deref_node,
             "__field",
-            src_offset=[offset for (_, offset) in arg_expr.field_domain],
         )
 
         for dim, index_expr in field_indices:
@@ -657,12 +700,12 @@ class LambdaToDataflow(eve.NodeVisitor):
                 # structures more explicit and thus makes it easier for MapFusion
                 # to correctly infer the data dependencies.
                 memlet_subset = arg.get_memlet_subset(self.sdfg)
-                arg_expr = MemletExpr(arg.field, arg.gt_dtype, memlet_subset)
+                arg_expr = MemletExpr(arg.field, arg.get_field_type(), memlet_subset)
             else:
                 # In order to shift the iterator inside the branch dataflow,
                 # we have to pass the full array shape.
                 arg_expr = MemletExpr(
-                    arg.field, arg.gt_dtype, dace_subsets.Range.from_array(arg_desc)
+                    arg.field, arg.get_field_type(), dace_subsets.Range.from_array(arg_desc)
                 )
                 use_full_shape = True
         else:
@@ -680,7 +723,9 @@ class LambdaToDataflow(eve.NodeVisitor):
                 arg.gt_dtype.offset_type.value
             )
             assert isinstance(offset_provider_type, gtx_common.NeighborConnectivityType)
-            inner_desc = dace.data.Array(arg_desc.dtype, [offset_provider_type.max_neighbors])
+            inner_desc = dace.data.Array(
+                dtype=arg_desc.dtype, shape=[offset_provider_type.max_neighbors]
+            )
 
         if param_name in if_sdfg.arrays:
             # the data desciptor was added by the visitor of the other branch expression
@@ -843,7 +888,7 @@ class LambdaToDataflow(eve.NodeVisitor):
 
         # create states inside the nested SDFG for the if-branches
         if_region = dace.sdfg.state.ConditionalBlock("if")
-        nsdfg.add_node(if_region)
+        nsdfg.add_node(if_region, ensure_unique_name=True)
         entry_state = nsdfg.add_state("entry", is_start_block=True)
         nsdfg.add_edge(entry_state, if_region, dace.InterstateEdge())
 
@@ -933,7 +978,6 @@ class LambdaToDataflow(eve.NodeVisitor):
             nsdfg_symbols_mapping["__cond"] = condition_value.value
         nsdfg_node = self.state.add_nested_sdfg(
             nsdfg,
-            self.sdfg,
             inputs=set(input_memlets.keys()),
             outputs=outputs,
             symbol_mapping=nsdfg_symbols_mapping,
@@ -994,7 +1038,7 @@ class LambdaToDataflow(eve.NodeVisitor):
         field_slice = self._construct_local_view(
             MemletExpr(
                 dc_node=it.field,
-                gt_dtype=node.type,
+                gt_field=it.get_field_type(),
                 subset=dace_subsets.Range.from_string(
                     ",".join(
                         str(it.indices[dim].value - offset)  # type: ignore[union-attr]
@@ -1007,10 +1051,16 @@ class LambdaToDataflow(eve.NodeVisitor):
                 ),
             )
         )
+        # The layout of connectivity tables is known.
+        assert len(offset_provider.domain) == 2
+        assert offset_provider.domain[1].kind == gtx_common.DimensionKind.LOCAL
         connectivity_slice = self._construct_local_view(
             MemletExpr(
                 dc_node=self.state.add_access(connectivity),
-                gt_dtype=node.type,
+                gt_field=ts.FieldType(
+                    dims=[offset_provider.domain[0]],
+                    dtype=ts.ListType(element_type=_INDEX_DTYPE, offset_type=_CONST_DIM),
+                ),
                 subset=dace_subsets.Range.from_string(
                     f"{origin_index.value}, 0:{offset_provider.max_neighbors}"
                 ),
@@ -1025,10 +1075,11 @@ class LambdaToDataflow(eve.NodeVisitor):
         neighbor_idx = gtir_to_sdfg_utils.get_map_variable(offset_type)
 
         index_connector = "__index"
+        field_connector = "__field"
         output_connector = "__val"
-        tasklet_expression = f"{output_connector} = __field[{index_connector}]"
+        tasklet_expression = f"{output_connector} = {field_connector}[{index_connector}]"
         input_memlets = {
-            "__field": self.sdfg.make_array_memlet(field_slice.dc_node.data),
+            field_connector: self.sdfg.make_array_memlet(field_slice.dc_node.data),
             index_connector: dace.Memlet(data=connectivity_slice.dc_node.data, subset=neighbor_idx),
         }
         input_nodes = {
@@ -1067,27 +1118,45 @@ class LambdaToDataflow(eve.NodeVisitor):
     def _visit_list_get(self, node: gtir.FunCall) -> ValueExpr:
         assert len(node.args) == 2
         index_arg = self.visit(node.args[0])
-        list_arg = self.visit(node.args[1])
-        assert isinstance(list_arg, ValueExpr)
-        assert isinstance(list_arg.gt_dtype, ts.ListType)
-        assert isinstance(list_arg.gt_dtype.element_type, ts.ScalarType)
+        src_arg = self.visit(node.args[1])
+        assert isinstance(src_arg.gt_dtype, ts.ListType)
 
-        list_desc = list_arg.dc_node.desc(self.sdfg)
-        assert len(list_desc.shape) == 1
+        src_desc = src_arg.dc_node.desc(self.sdfg)
+        if isinstance(src_arg, MemletExpr):
+            assert len(src_desc.shape) == len(src_arg.subset)
+            src_subset = src_arg.subset
+            local_dim = src_arg.gt_dtype.offset_type
+            full_dims = gtx_common.order_dimensions([*src_arg.gt_field.dims, local_dim])  # type: ignore[list-item]  # checked in MemletExpr.__post_init__
+            local_dim_index = full_dims.index(local_dim)  # type: ignore[arg-type]  # checked in MemletExpr.__post_init__
+        elif isinstance(src_arg, ValueExpr):
+            assert len(src_desc.shape) == 1
+            src_subset = dace_subsets.Range.from_array(src_desc)
+            local_dim_index = 0
+        else:
+            raise ValueError(f"Unexpected argument type {type(src_arg)} in 'list_get' expression.")
 
-        result_dtype = gtx_dace_utils.as_dace_type(list_arg.gt_dtype.element_type)
-        result, _ = self.subgraph_builder.add_temp_scalar(self.sdfg, result_dtype)
-        result_node = self.state.add_access(result)
+        assert isinstance(src_arg.gt_dtype.element_type, ts.ScalarType)
+        assert src_desc.dtype == gtx_dace_utils.as_dace_type(src_arg.gt_dtype.element_type)
+        dst, _ = self.subgraph_builder.add_temp_scalar(self.sdfg, src_desc.dtype)
+        dst_node = self.state.add_access(dst)
 
         if isinstance(index_arg, SymbolExpr):
             assert index_arg.dc_dtype in dace.dtypes.INTEGER_TYPES
-            self._add_edge(
-                list_arg.dc_node,
-                None,
-                result_node,
-                None,
-                dace.Memlet(data=list_arg.dc_node.data, subset=index_arg.value),
+            src_subset = (
+                dace_subsets.Range(src_subset[:local_dim_index])
+                + dace_subsets.Range.from_string(index_arg.value)
+                + dace_subsets.Range(src_subset[local_dim_index + 1 :])
             )
+            if isinstance(src_arg, MemletExpr):
+                self._add_input_data_edge(src_arg.dc_node, src_subset, dst_node)
+            else:
+                self._add_edge(
+                    src_arg.dc_node,
+                    None,
+                    dst_node,
+                    None,
+                    dace.Memlet(data=src_arg.dc_node.data, subset=src_subset, other_subset="0"),
+                )
         elif isinstance(index_arg, ValueExpr):
             tasklet_node = self._add_tasklet(
                 "list_get",
@@ -1102,20 +1171,15 @@ class LambdaToDataflow(eve.NodeVisitor):
                 "__index",
                 dace.Memlet(data=index_arg.dc_node.data, subset="0"),
             )
-            self._add_edge(
-                list_arg.dc_node,
-                None,
-                tasklet_node,
-                "__data",
-                self.sdfg.make_array_memlet(list_arg.dc_node.data),
-            )
-            self._add_edge(
-                tasklet_node, "__val", result_node, None, dace.Memlet(data=result, subset="0")
-            )
+            if isinstance(src_arg, MemletExpr):
+                self._add_input_data_edge(src_arg.dc_node, src_subset, tasklet_node, "__data")
+            else:
+                self._add_edge(src_arg.dc_node, None, tasklet_node, "__data", src_subset)
+            self._add_edge(tasklet_node, "__val", dst_node, None, dace.Memlet(data=dst, subset="0"))
         else:
             raise TypeError(f"Unexpected value {index_arg} as index argument.")
 
-        return ValueExpr(dc_node=result_node, gt_dtype=list_arg.gt_dtype.element_type)
+        return ValueExpr(dc_node=dst_node, gt_dtype=src_arg.gt_dtype.element_type)
 
     def _visit_map(self, node: gtir.FunCall) -> ValueExpr:
         """
@@ -1220,11 +1284,15 @@ class LambdaToDataflow(eve.NodeVisitor):
 
             origin_map_index = gtir_to_sdfg_utils.get_map_variable(offset_provider_type.source_dim)
 
+            # The layout of connectivity tables is known.
+            assert len(offset_provider_type.domain) == 2
+            assert offset_provider_type.domain[1].kind == gtx_common.DimensionKind.LOCAL
             connectivity_slice = self._construct_local_view(
                 MemletExpr(
                     dc_node=self.state.add_access(connectivity),
-                    gt_dtype=ts.ListType(
-                        element_type=node.type.element_type, offset_type=offset_type
+                    gt_field=ts.FieldType(
+                        dims=[offset_provider_type.domain[0]],
+                        dtype=ts.ListType(element_type=_INDEX_DTYPE, offset_type=_CONST_DIM),
                     ),
                     subset=dace_subsets.Range.from_string(
                         f"{origin_map_index}, 0:{offset_provider_type.max_neighbors}"
@@ -1361,7 +1429,7 @@ class LambdaToDataflow(eve.NodeVisitor):
         )
 
         nsdfg_node = self.state.add_nested_sdfg(
-            nsdfg, self.sdfg, inputs={"values", "neighbor_indices"}, outputs={"acc"}
+            nsdfg, inputs={"values", "neighbor_indices"}, outputs={"acc"}
         )
 
         if isinstance(input_expr, MemletExpr):
@@ -1374,6 +1442,9 @@ class LambdaToDataflow(eve.NodeVisitor):
                 "values",
                 self.sdfg.make_array_memlet(input_expr.dc_node.data),
             )
+        # The layout of connectivity tables is known.
+        assert len(offset_provider_type.domain) == 2
+        assert offset_provider_type.domain[1].kind == gtx_common.DimensionKind.LOCAL
         self._add_input_data_edge(
             connectivity_node,
             dace_subsets.Range.from_string(
@@ -1592,7 +1663,10 @@ class LambdaToDataflow(eve.NodeVisitor):
             # use memlet to retrieve the neighbor index
             shifted_indices[neighbor_dim] = MemletExpr(
                 dc_node=offset_table_node,
-                gt_dtype=it.gt_dtype,
+                gt_field=ts.FieldType(
+                    dims=[origin_dim],
+                    dtype=ts.ListType(element_type=_INDEX_DTYPE, offset_type=_CONST_DIM),
+                ),
                 subset=dace_subsets.Range.from_string(f"{origin_index.value}, {offset_expr.value}"),
             )
         else:
@@ -1604,11 +1678,6 @@ class LambdaToDataflow(eve.NodeVisitor):
         return IteratorExpr(it.field, it.gt_dtype, it.field_domain, shifted_indices)
 
     def _visit_shift(self, node: gtir.FunCall) -> IteratorExpr:
-        # convert builtin-index type to dace type
-        IndexDType: Final = gtx_dace_utils.as_dace_type(
-            ts.ScalarType(kind=getattr(ts.ScalarKind, gtir_builtins.INTEGER_INDEX_BUILTIN.upper()))
-        )
-
         assert isinstance(node.fun, gtir.FunCall)
         # the iterator to be shifted is the node argument, while the shift arguments
         # are provided by the nested function call; the shift arguments consist of
@@ -1624,7 +1693,7 @@ class LambdaToDataflow(eve.NodeVisitor):
         offset_provider_type = self.subgraph_builder.get_offset_provider_type(offset)
         # second argument should be the offset value, which could be a symbolic expression or a dynamic offset
         offset_expr = (
-            SymbolExpr(offset_value_arg.value, IndexDType)
+            SymbolExpr(offset_value_arg.value, gtx_dace_utils.as_dace_type(_INDEX_DTYPE))
             if isinstance(offset_value_arg, gtir.OffsetLiteral)
             else self.visit(offset_value_arg)
         )
@@ -1654,7 +1723,7 @@ class LambdaToDataflow(eve.NodeVisitor):
             arg_expr = self.visit(arg)
             if isinstance(arg_expr, MemletExpr | ValueExpr):
                 # the argument value is the result of a tasklet node or direct field access
-                connector = f"__arg{i}"
+                connector = f"__var{i}"
                 node_connections[connector] = arg_expr
                 node_internals.append(connector)
             else:
@@ -1712,7 +1781,9 @@ class LambdaToDataflow(eve.NodeVisitor):
             dc_dtype = gtx_dace_utils.as_dace_type(node.type)
             use_array = False
 
-        return self._construct_tasklet_result(dc_dtype, tasklet_node, "result", use_array=use_array)
+        return self._construct_tasklet_result(
+            dc_dtype, tasklet_node, out_connector, use_array=use_array
+        )
 
     def _visit_make_tuple(self, node: gtir.FunCall) -> tuple[IteratorExpr | DataExpr]:
         assert cpm.is_call_to(node, "make_tuple")
