@@ -46,7 +46,6 @@ from gt4py.next.program_processors.runners.dace import (
     gtir_to_sdfg_primitives,
     gtir_to_sdfg_types,
     gtir_to_sdfg_utils,
-    transformations as gtx_transformations,
     utils as gtx_dace_utils,
 )
 from gt4py.next.type_system import type_specifications as ts, type_translation as tt
@@ -129,6 +128,15 @@ class DataflowBuilder(Protocol):
         return state.add_mapped_tasklet(unique_name, map_ranges, inputs, code, outputs, **kwargs)
 
 
+@dataclasses.dataclass(frozen=True)
+class SubgraphContext:
+    """Represents the subgraph context in which to lower a GTIR expression to dataflow."""
+
+    sdfg: dace.SDFG
+    state: dace.SDFGState
+    target_domain: gtir_domain.FieldopDomain
+
+
 class SDFGBuilder(DataflowBuilder, Protocol):
     """Visitor interface available to GTIR-primitive translators."""
 
@@ -155,11 +163,11 @@ class SDFGBuilder(DataflowBuilder, Protocol):
     def setup_nested_context(
         self,
         expr: gtir.Lambda,
-        sdfg: dace.SDFG,
-        parent: dace.SDFG,
+        sdfg_name: str,
+        parent_ctx: SubgraphContext,
         scope_symbols: dict[str, ts.DataType],
         symbolic_inputs: set[str],
-    ) -> SDFGBuilder:
+    ) -> tuple[SDFGBuilder, SubgraphContext]:
         """
         Create an nested SDFG context to lower a lambda expression, indipendent
         from the current context where the parent expression is being translated.
@@ -171,8 +179,8 @@ class SDFGBuilder(DataflowBuilder, Protocol):
 
         Args:
             expr: The lambda expression to be lowered as a nested SDFG.
-            sdfg: The nested SDFG where to lower the nested expression.
-            parent: The parent SDFG.
+            sdfg_name: The name of the nested SDFG where to lower the given expression.
+            parent_ctx: The parent SDFG context.
             scope_symbols: Mapping from symbol name to data type for the GTIR symbols
                 forwarded to the nested context.
             symbolic_inputs: Arguments that have to be passed to the nested SDFG
@@ -234,7 +242,7 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
     )
 
     def get_offset_provider_type(self, offset: str) -> gtx_common.OffsetProviderTypeElem:
-        return self.offset_provider_type[offset]
+        return gtx_common.get_offset_type(self.offset_provider_type, offset)
 
     def make_field(
         self,
@@ -268,27 +276,24 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
             field_type = data_type
         elif len(local_dims) == 1:
             local_dim = local_dims[0]
-            local_dim_index = data_type.dims.index(local_dim)
             # the local dimension is converted into `ListType` data element
             if not isinstance(data_type.dtype, ts.ScalarType):
                 raise ValueError(f"Invalid field type {data_type}.")
-            if local_dim_index != (len(data_type.dims) - 1):
-                raise ValueError(
-                    f"Invalid field domain: expected the local dimension to be at the end, found at position {local_dim_index}."
-                )
-            if local_dim.value not in self.offset_provider_type:
+            if not gtx_common.has_offset(self.offset_provider_type, local_dim.value):
                 raise ValueError(
                     f"The provided local dimension {local_dim} does not match any offset provider type."
                 )
             local_type = ts.ListType(element_type=data_type.dtype, offset_type=local_dim)
-            field_type = ts.FieldType(dims=data_type.dims[:local_dim_index], dtype=local_type)
+            field_type = ts.FieldType(
+                dims=[dim for dim in data_type.dims if dim != local_dim], dtype=local_type
+            )
         else:
             raise NotImplementedError(
                 "Fields with more than one local dimension are not supported."
             )
         field_origin = tuple(
-            dace.symbolic.pystr_to_symbolic(gtx_dace_utils.range_start_symbol(data_node.data, axis))
-            for axis in range(len(field_type.dims))
+            dace.symbolic.pystr_to_symbolic(gtx_dace_utils.range_start_symbol(data_node.data, dim))
+            for dim in field_type.dims
         )
         return gtir_to_sdfg_types.FieldopData(data_node, field_type, field_origin)
 
@@ -302,12 +307,17 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
     def setup_nested_context(
         self,
         expr: gtir.Lambda,
-        sdfg: dace.SDFG,
-        parent: dace.SDFG,
+        sdfg_name: str,
+        parent_ctx: SubgraphContext,
         scope_symbols: dict[str, ts.DataType],
         symbolic_inputs: set[str],
-    ) -> SDFGBuilder:
+    ) -> tuple[SDFGBuilder, SubgraphContext]:
+        sdfg = dace.SDFG(name=self.unique_nsdfg_name(parent_ctx.sdfg, sdfg_name))
+        sdfg.debuginfo = gtir_to_sdfg_utils.debug_info(expr, default=parent_ctx.sdfg.debuginfo)
+        state = sdfg.add_state(f"{sdfg_name}_entry")
+        nested_ctx = SubgraphContext(sdfg, state, parent_ctx.target_domain)
         nsdfg_builder = GTIRToSDFG(self.offset_provider_type, self.column_axis, scope_symbols)
+
         # We pass to the nested SDFG all GTIR-symbols in scope, which includes the
         # values mapped to the parameters of the lambda expression (`node.params`)
         # and the GTIR-symbols defined in the current context.
@@ -331,7 +341,9 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
         # Scalar GTIR-symbols represented as dace symbols in parent SDFG are mapped
         # to dace symbols in the nested SDFG.
         parent_symbols = {
-            name for p in flatten_tuple_syms(params) if (name := str(p.id)) in parent.symbols
+            name
+            for p in flatten_tuple_syms(params)
+            if (name := str(p.id)) in parent_ctx.sdfg.symbols
         }
 
         # All GTIR-symbols accessed in domain expressions by the lambda need to be
@@ -354,7 +366,7 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
             params,
             symbolic_params=(domain_symbols | parent_symbols | symbolic_inputs),
         )
-        return nsdfg_builder
+        return nsdfg_builder, nested_ctx
 
     def unique_nsdfg_name(self, sdfg: dace.SDFG, prefix: str) -> str:
         nsdfg_list = [
@@ -402,8 +414,8 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
                 shape.append(
                     dace.symbolic.pystr_to_symbolic(
                         "{} - {}".format(
-                            gtx_dace_utils.range_stop_symbol(name, i),
-                            gtx_dace_utils.range_start_symbol(name, i),
+                            gtx_dace_utils.range_stop_symbol(name, dim),
+                            gtx_dace_utils.range_start_symbol(name, dim),
                         )
                     )
                 )
@@ -474,40 +486,30 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
                     gt_type=gt_type.dtype,
                     transient=transient,
                 )
-            if not isinstance(gt_type.dtype, ts.ScalarType):
-                raise ValueError(f"Field type '{gt_type.dtype}' not supported.")
-            # handle default case: field with one or more dimensions
-            dc_dtype = gtx_dace_utils.as_dace_type(gt_type.dtype)
+            if isinstance(gt_type.dtype, ts.ScalarType):
+                dc_dtype = gtx_dace_utils.as_dace_type(gt_type.dtype)
+                dims = gt_type.dims
+            elif not transient:  # 'ts.ListType': use 'offset_type' as local dimension
+                assert gt_type.dtype.offset_type is not None
+                assert gt_type.dtype.offset_type.kind == gtx_common.DimensionKind.LOCAL
+                assert isinstance(gt_type.dtype.element_type, ts.ScalarType)
+                dc_dtype = gtx_dace_utils.as_dace_type(gt_type.dtype.element_type)
+                dims = gtx_common.order_dimensions([*gt_type.dims, gt_type.dtype.offset_type])
+            else:
+                # By design, the domain of temporary fields used by SDFG lowering
+                # contains only the global dimensions. The local dimension is extracted,
+                # when needed, from the GTIR data type (`ts.ListType`).
+                raise ValueError("Unexpected local dimension in temporary field domain.")
             # Use symbolic shape, which allows to invoke the program with fields of different size;
             # and symbolic strides, which enables decoupling the memory layout from generated code.
-            sym_shape, sym_strides = self._make_array_shape_and_strides(name, gt_type.dims)
+            sym_shape, sym_strides = self._make_array_shape_and_strides(name, dims)
             sdfg.add_array(name, sym_shape, dc_dtype, strides=sym_strides, transient=transient)
             return [(name, gt_type)]
 
         elif isinstance(gt_type, ts.ScalarType):
             dc_dtype = gtx_dace_utils.as_dace_type(gt_type)
-            if (
-                gtx_dace_utils.is_range_symbol(name)
-                or symbolic_params is None
-                or name in symbolic_params
-            ):
-                if name in sdfg.symbols:
-                    # Sometimes, when the field domain is implicitly derived from
-                    # the field shape, the gt4py lowering adds the field domain
-                    # range symbols as scalar arguments to the program IR.
-                    # Suppose a field '__sym', then gt4py will add '__sym_0_range_0'
-                    # and '__sym_0_range_1'.
-                    # Therefore, here we check whether the shape symbol was already
-                    # created by `_make_array_shape_and_strides()`, when allocating
-                    # storage for field arguments. We assume that the scalar argument
-                    # for field domain, if present, always follows the field argument.
-                    assert gtx_dace_utils.is_range_symbol(name)
-                    if sdfg.symbols[name].dtype != dc_dtype:
-                        raise ValueError(
-                            f"Type mismatch on argument {name}: got {dc_dtype}, expected {sdfg.symbols[name].dtype}."
-                        )
-                else:
-                    sdfg.add_symbol(name, dc_dtype)
+            if symbolic_params is None or name in symbolic_params:
+                sdfg.add_symbol(name, dc_dtype)
             else:
                 sdfg.add_scalar(name, dc_dtype, transient=transient)
 
@@ -524,7 +526,12 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
         raise NotImplementedError("Temporaries not supported yet by GTIR DaCe backend.")
 
     def _visit_expression(
-        self, node: gtir.Expr, sdfg: dace.SDFG, head_state: dace.SDFGState, use_temp: bool = True
+        self,
+        node: gtir.Expr,
+        domain: gtir_domain.FieldopDomain,
+        sdfg: dace.SDFG,
+        head_state: dace.SDFGState,
+        use_temp: bool = True,
     ) -> list[gtir_to_sdfg_types.FieldopData]:
         """
         Specialized visit method for fieldview expressions.
@@ -535,7 +542,9 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
         Returns:
             A list of array nodes containing the result fields.
         """
-        result = self.visit(node, sdfg=sdfg, head_state=head_state)
+
+        ctx = SubgraphContext(sdfg, head_state, domain)
+        result = self.visit(node, ctx=ctx)
 
         # sanity check: each statement should preserve the property of single exit state (aka head state),
         # i.e. eventually only introduce internal branches, and keep the same head state
@@ -686,14 +695,13 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
           The SDFG head state, eventually updated if the target write requires a new state.
         """
 
-        source_fields = self._visit_expression(stmt.expr, sdfg, state)
+        # visit the domain expression
+        domain = gtir_domain.extract_domain(stmt.domain)
+        source_fields = self._visit_expression(stmt.expr, domain, sdfg, state)
 
         # the target expression could be a `SymRef` to an output node or a `make_tuple` expression
         # in case the statement returns more than one field
-        target_fields = self._visit_expression(stmt.target, sdfg, state, use_temp=False)
-
-        # visit the domain expression
-        domain = gtir_domain.extract_domain(stmt.domain)
+        target_fields = self._visit_expression(stmt.target, domain, sdfg, state, use_temp=False)
 
         expr_input_args = {
             sym_id
@@ -744,22 +752,21 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
     def visit_FunCall(
         self,
         node: gtir.FunCall,
-        sdfg: dace.SDFG,
-        head_state: dace.SDFGState,
+        ctx: SubgraphContext,
     ) -> gtir_to_sdfg_types.FieldopResult:
         # use specialized dataflow builder classes for each builtin function
         if cpm.is_call_to(node, "concat_where"):
-            return gtir_to_sdfg_concat_where.translate_concat_where(node, sdfg, head_state, self)
+            return gtir_to_sdfg_concat_where.translate_concat_where(node, ctx, self)
         elif cpm.is_call_to(node, "if_"):
-            return gtir_to_sdfg_primitives.translate_if(node, sdfg, head_state, self)
+            return gtir_to_sdfg_primitives.translate_if(node, ctx, self)
         elif cpm.is_call_to(node, "index"):
-            return gtir_to_sdfg_primitives.translate_index(node, sdfg, head_state, self)
+            return gtir_to_sdfg_primitives.translate_index(node, ctx, self)
         elif cpm.is_call_to(node, "make_tuple"):
-            return gtir_to_sdfg_primitives.translate_make_tuple(node, sdfg, head_state, self)
+            return gtir_to_sdfg_primitives.translate_make_tuple(node, ctx, self)
         elif cpm.is_call_to(node, "tuple_get"):
-            return gtir_to_sdfg_primitives.translate_tuple_get(node, sdfg, head_state, self)
+            return gtir_to_sdfg_primitives.translate_tuple_get(node, ctx, self)
         elif cpm.is_applied_as_fieldop(node):
-            return gtir_to_sdfg_primitives.translate_as_fieldop(node, sdfg, head_state, self)
+            return gtir_to_sdfg_primitives.translate_as_fieldop(node, ctx, self)
         elif isinstance(node.fun, gtir.Lambda):
             # Special handling of scalar arguments of a let-lambda that can be lowered
             # as symbolic expressions: when all the GTIR-symbols the scalar expression
@@ -780,37 +787,27 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
                     ):
                         raise
                     continue
-                if all(str(s) in sdfg.symbols for s in symbolic_expr.free_symbols):
+                if all(str(s) in ctx.sdfg.symbols for s in symbolic_expr.free_symbols):
                     symbolic_args[str(p.id)] = symbolic_expr
             # All other lambda arguments are lowered to some dataflow that produces a data node.
             args = {
                 str(p.id): (
                     gtir_to_sdfg_types.SymbolicData(p.type, symbolic_args[param])  # type: ignore[arg-type]
                     if (param := str(p.id)) in symbolic_args
-                    else self.visit(
-                        arg,
-                        sdfg=sdfg,
-                        head_state=head_state,
-                    )
+                    else self.visit(arg, ctx=ctx)
                 )
                 for p, arg in zip(node.fun.params, node.args, strict=True)
             }
-            return self.visit(
-                node.fun,
-                sdfg=sdfg,
-                head_state=head_state,
-                args=args,
-            )
+            return self.visit(node.fun, ctx=ctx, args=args)
         elif isinstance(node.type, ts.ScalarType):
-            return gtir_to_sdfg_primitives.translate_scalar_expr(node, sdfg, head_state, self)
+            return gtir_to_sdfg_primitives.translate_scalar_expr(node, ctx, self)
         else:
             raise NotImplementedError(f"Unexpected 'FunCall' expression ({node}).")
 
     def visit_Lambda(
         self,
         node: gtir.Lambda,
-        sdfg: dace.SDFG,
-        head_state: dace.SDFGState,
+        ctx: SubgraphContext,
         args: Mapping[str, gtir_to_sdfg_types.FieldopResult | gtir_to_sdfg_types.SymbolicData],
     ) -> gtir_to_sdfg_types.FieldopResult:
         """
@@ -859,18 +856,11 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
         }
 
         # lower let-statement lambda node as a nested SDFG
-        nsdfg = dace.SDFG(name=self.unique_nsdfg_name(sdfg, "lambda"))
-        nsdfg.debuginfo = gtir_to_sdfg_utils.debug_info(node, default=sdfg.debuginfo)
-        lambda_translator = self.setup_nested_context(
-            node, nsdfg, sdfg, lambda_symbols, symbolic_inputs=set(symbolic_args.keys())
+        lamnda_translator, lambda_ctx = self.setup_nested_context(
+            node, "lambda", ctx, lambda_symbols, symbolic_inputs=set(symbolic_args.keys())
         )
 
-        nstate = nsdfg.add_state("lambda")
-        lambda_result = lambda_translator.visit(
-            node.expr,
-            sdfg=nsdfg,
-            head_state=nstate,
-        )
+        lambda_result = lamnda_translator.visit(node.expr, ctx=lambda_ctx)
 
         # Process lambda inputs
         #
@@ -883,23 +873,23 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
         }
 
         input_memlets = {}
-        for nsdfg_dataname, nsdfg_datadesc in nsdfg.arrays.items():
+        for nsdfg_dataname, nsdfg_datadesc in lambda_ctx.sdfg.arrays.items():
             if nsdfg_datadesc.transient:
                 continue
 
             if nsdfg_dataname in lambda_arg_nodes:
                 src_node = lambda_arg_nodes[nsdfg_dataname].dc_node
                 dataname = src_node.data
-                datadesc = src_node.desc(sdfg)
+                datadesc = src_node.desc(ctx.sdfg)
             else:
                 dataname = nsdfg_dataname
-                datadesc = sdfg.arrays[nsdfg_dataname]
+                datadesc = ctx.sdfg.arrays[nsdfg_dataname]
 
             # ensure that connectivity tables are non-transient arrays in parent SDFG
             if dataname in connectivity_arrays:
                 datadesc.transient = False
 
-            input_memlets[nsdfg_dataname] = sdfg.make_array_memlet(dataname)
+            input_memlets[nsdfg_dataname] = ctx.sdfg.make_array_memlet(dataname)
 
         # Process lambda outputs
         #
@@ -925,12 +915,12 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
         lambda_outputs = {
             output_data.dc_node.data
             for output_data in lambda_output_data
-            if output_data.dc_node.desc(nsdfg).transient
+            if output_data.dc_node.desc(lambda_ctx.sdfg).transient
         }
 
         # Map free symbols to parent SDFG
         nsdfg_symbols_mapping = {}
-        for sym in nsdfg.free_symbols:
+        for sym in lambda_ctx.sdfg.free_symbols:
             if (sym_id := str(sym)) in lambda_arg_nodes:
                 assert isinstance(lambda_arg_nodes[sym_id].gt_type, ts.ScalarType)
                 raise NotImplementedError(
@@ -941,24 +931,23 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
             else:
                 nsdfg_symbols_mapping[sym_id] = sym
         for param, arg in data_args.items():
-            nsdfg_symbols_mapping |= gtir_to_sdfg_utils.get_arg_symbol_mapping(param, arg, sdfg)
+            nsdfg_symbols_mapping |= gtir_to_sdfg_utils.get_arg_symbol_mapping(param, arg, ctx.sdfg)
 
-        nsdfg_node = head_state.add_nested_sdfg(
-            nsdfg,
-            parent=sdfg,
+        nsdfg_node = ctx.state.add_nested_sdfg(
+            lambda_ctx.sdfg,
             inputs=set(input_memlets.keys()),
             outputs=lambda_outputs,
             symbol_mapping=nsdfg_symbols_mapping,
-            debuginfo=gtir_to_sdfg_utils.debug_info(node, default=sdfg.debuginfo),
+            debuginfo=gtir_to_sdfg_utils.debug_info(node, default=ctx.sdfg.debuginfo),
         )
 
         for connector, memlet in input_memlets.items():
             if connector in lambda_arg_nodes:
                 src_node = lambda_arg_nodes[connector].dc_node
             else:
-                src_node = head_state.add_access(memlet.data)
+                src_node = ctx.state.add_access(memlet.data)
 
-            head_state.add_edge(src_node, None, nsdfg_node, connector, memlet)
+            ctx.state.add_edge(src_node, None, nsdfg_node, connector, memlet)
 
         def construct_output_for_nested_sdfg(
             inner_data: gtir_to_sdfg_types.FieldopData,
@@ -975,7 +964,7 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
             The same happens to symbols available in the lambda context but not explicitly passed as lambda
             arguments, that are simply returned by the lambda: it can be directly accessed in the parent SDFG.
             """
-            inner_desc = inner_data.dc_node.desc(nsdfg)
+            inner_desc = inner_data.dc_node.desc(lambda_ctx.sdfg)
             inner_dataname = inner_data.dc_node.data
             if inner_desc.transient:
                 # Transient data nodes only exist within the nested SDFG. In order to return some result data,
@@ -983,14 +972,14 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
                 # that is externally allocated, as required by the SDFG IR. An output edge will write the result
                 # from the nested-SDFG to a new intermediate data container allocated in the parent SDFG.
                 outer_data = inner_data.map_to_parent_sdfg(
-                    self, nsdfg, sdfg, head_state, nsdfg_symbols_mapping
+                    self, lambda_ctx.sdfg, ctx.sdfg, ctx.state, nsdfg_symbols_mapping
                 )
-                head_state.add_edge(
+                ctx.state.add_edge(
                     nsdfg_node,
                     inner_dataname,
                     outer_data.dc_node,
                     None,
-                    sdfg.make_array_memlet(outer_data.dc_node.data),
+                    ctx.sdfg.make_array_memlet(outer_data.dc_node.data),
                 )
             elif inner_dataname in lambda_arg_nodes:
                 # This if branch and the next one handle the non-transient result nodes.
@@ -1000,15 +989,15 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
                 outer_data = lambda_arg_nodes[inner_dataname]
             else:
                 # This must be a symbol captured from the lambda parent scope.
-                outer_node = head_state.add_access(inner_dataname)
+                outer_node = ctx.state.add_access(inner_dataname)
                 outer_data = gtir_to_sdfg_types.FieldopData(
                     outer_node, inner_data.gt_type, inner_data.origin
                 )
             # Isolated access node will make validation fail.
             # Isolated access nodes can be found in the join-state of an if-expression
             # or in lambda expressions that just construct tuples from input arguments.
-            if nstate.degree(inner_data.dc_node) == 0:
-                nstate.remove_node(inner_data.dc_node)
+            if lambda_ctx.state.degree(inner_data.dc_node) == 0:
+                lambda_ctx.state.remove_node(inner_data.dc_node)
             return outer_data
 
         return gtx_utils.tree_map(construct_output_for_nested_sdfg)(lambda_result)
@@ -1016,76 +1005,33 @@ class GTIRToSDFG(eve.NodeVisitor, SDFGBuilder):
     def visit_Literal(
         self,
         node: gtir.Literal,
-        sdfg: dace.SDFG,
-        head_state: dace.SDFGState,
+        ctx: SubgraphContext,
     ) -> gtir_to_sdfg_types.FieldopResult:
-        return gtir_to_sdfg_primitives.translate_literal(node, sdfg, head_state, self)
+        return gtir_to_sdfg_primitives.translate_literal(node, ctx, self)
 
     def visit_SymRef(
         self,
         node: gtir.SymRef,
-        sdfg: dace.SDFG,
-        head_state: dace.SDFGState,
+        ctx: SubgraphContext,
     ) -> gtir_to_sdfg_types.FieldopResult:
-        return gtir_to_sdfg_primitives.translate_symbol_ref(node, sdfg, head_state, self)
-
-
-def _remove_field_origin_symbols(ir: gtir.Program, sdfg: dace.SDFG) -> None:
-    """
-    Helper function to remove the origin symbols used in program field arguments,
-    that is only for non-transient data descriptors in the top-level SDFG.
-    The start symbol of field domain range is set to constant value 0, thus removing
-    the corresponding free symbol. These values are propagated to all nested SDFGs.
-
-    This function is only used by `build_sdfg_from_gtir()` when the option flag
-    `disable_field_origin_on_program_arguments` is set to True.
-    """
-
-    # collect symbols used as range start for all program arguments
-    range_start_symbols: dict[str, dace.symbolic.SymExpr] = {}
-    for p in ir.params:
-        if isinstance(p.type, ts.TupleType):
-            psymbols = [
-                sym
-                for sym in gtir_to_sdfg_utils.flatten_tuple_fields(p.id, p.type)
-                if isinstance(sym.type, ts.FieldType)
-            ]
-        elif isinstance(p.type, ts.FieldType):
-            psymbols = [p]
-        else:
-            psymbols = []
-        for psymbol in psymbols:
-            assert isinstance(psymbol.type, ts.FieldType)
-            if len(psymbol.type.dims) == 0:
-                # zero-dimensional field
-                continue
-            dataname = str(psymbol.id)
-            # set all range start symbols to constant value 0
-            range_start_symbols |= {
-                gtx_dace_utils.range_start_symbol(dataname, i): 0
-                for i in range(len(psymbol.type.dims))
-            }
-    # we set all range start symbols to 0 in the top-level SDFG and proagate them to nested SDFGs
-    gtx_transformations.gt_substitute_compiletime_symbols(sdfg, range_start_symbols, validate=True)
+        return gtir_to_sdfg_primitives.translate_symbol_ref(node, ctx, self)
 
 
 def build_sdfg_from_gtir(
     ir: gtir.Program,
     offset_provider_type: gtx_common.OffsetProviderType,
     column_axis: Optional[gtx_common.Dimension] = None,
-    disable_field_origin_on_program_arguments: bool = False,
 ) -> dace.SDFG:
     """
     Receives a GTIR program and lowers it to a DaCe SDFG.
 
-    The lowering to SDFG requires that the program node is type-annotated, therefore this function
-    runs type ineference as first step.
+    The lowering to SDFG requires that the program node is type-annotated, therefore
+    this function runs type ineference as first step.
 
     Args:
         ir: The GTIR program node to be lowered to SDFG
         offset_provider_type: The definitions of offset providers used by the program node
         column_axis: Vertical dimension used for column scan expressions.
-        disable_field_origin_on_program_arguments: When True, the field range in all dimensions is assumed to start from 0
 
     Returns:
         An SDFG in the DaCe canonical form (simplified)
@@ -1108,7 +1054,4 @@ def build_sdfg_from_gtir(
     sdfg = sdfg_genenerator.visit(ir)
     assert isinstance(sdfg, dace.SDFG)
 
-    if disable_field_origin_on_program_arguments:
-        _remove_field_origin_symbols(ir, sdfg)
-
     return sdfg
