@@ -32,6 +32,7 @@ def gt_gpu_transformation(
     gpu_block_size: Optional[Sequence[int | str] | str] = None,
     gpu_launch_bounds: Optional[int | str] = None,
     gpu_launch_factor: Optional[int] = None,
+    gpu_block_size_spec: Optional[dict[str, Sequence[int | str] | str]] = None,
     validate: bool = True,
     validate_all: bool = False,
     **kwargs: Any,
@@ -57,6 +58,8 @@ def gt_gpu_transformation(
             Will only take effect if `gpu_block_size` is specified.
         gpu_launch_factor: Use the number of threads times this value as `__launch_bounds__`
             Will only take effect if `gpu_block_size` is specified.
+        gpu_block_size_spec: Specify thread block size per dimension, see
+            `gt_set_gpu_blocksize()` for more.
         validate: Perform validation during the steps.
         validate_all: Perform extensive validation.
 
@@ -84,39 +87,43 @@ def gt_gpu_transformation(
 
     # Now turn it into a GPU SDFG
     sdfg.apply_gpu_transformations(
-        validate=validate,
+        validate=False,
         validate_all=validate_all,
         simplify=False,
     )
 
     # The documentation recommends to run simplify afterwards
-    gtx_transformations.gt_simplify(sdfg)
+    gtx_transformations.gt_simplify(sdfg, validate=False, validate_all=validate_all)
 
     if try_removing_trivial_maps:
         # TODO(phimuell): Figuring out if it is still important/needed to do or if
         #   it can be removed, it should definitely be reworked.
         gt_remove_trivial_gpu_maps(
             sdfg=sdfg,
-            validate=validate,
+            validate=False,
             validate_all=validate_all,
         )
-        gtx_transformations.gt_simplify(sdfg, validate=validate, validate_all=validate_all)
+        gtx_transformations.gt_simplify(sdfg, validate=False, validate_all=validate_all)
 
     # TODO(phimuell): Fixing the stride problem in DaCe.
     sdfg = gt_gpu_transform_non_standard_memlet(
         sdfg=sdfg,
         map_postprocess=True,
-        validate=validate,
+        validate=False,
         validate_all=validate_all,
     )
 
     # Set the GPU block size if it is known.
-    if gpu_block_size is not None:
+    if gpu_block_size is not None or gpu_block_size_spec is not None:
+        gpu_block_size_spec = gpu_block_size_spec or {}
         gt_set_gpu_blocksize(
             sdfg=sdfg,
             block_size=gpu_block_size,
             launch_bounds=gpu_launch_bounds,
             launch_factor=gpu_launch_factor,
+            **gpu_block_size_spec,
+            validate=False,
+            validate_all=validate_all,
         )
 
     if validate_all or validate:
@@ -188,6 +195,7 @@ def gt_gpu_transform_non_standard_memlet(
 
     # Now try to fuse the maps together, but restrict them that at least one map
     #  needs to be new.
+    # TODO(phimuell): Improve this by replacing it by an explicit loop.
     sdfg.apply_transformations_repeated(
         [
             gtx_transformations.MapFusionVertical(
@@ -199,7 +207,7 @@ def gt_gpu_transform_non_standard_memlet(
                 check_fusion_callback=restrict_fusion_to_newly_created_maps_horizontal,
             ),
         ],
-        validate=validate,
+        validate=False,
         validate_all=validate_all,
     )
 
@@ -242,6 +250,9 @@ def gt_gpu_transform_non_standard_memlet(
             )
         )
 
+    if validate or validate_all:
+        sdfg.validate()
+
     return sdfg
 
 
@@ -350,8 +361,10 @@ def gt_set_gpu_blocksize(
     block_size: Optional[Sequence[int | str] | str],
     launch_bounds: Optional[int | str] = None,
     launch_factor: Optional[int] = None,
+    validate: bool = True,
+    validate_all: bool = False,
     **kwargs: Any,
-) -> Any:
+) -> int:
     """Set the block size related properties of _all_ Maps.
 
     It supports the same arguments as `GPUSetBlockSize`, however it also has
@@ -364,6 +377,8 @@ def gt_set_gpu_blocksize(
         launch_bounds: The value for the launch bound that should be used.
         launch_factor: If no `launch_bounds` was given use the number of threads
             in a block multiplied by this number.
+        validate: Perform validation at the end of the function.
+        validate_all: Perform validation also on intermediate steps.
 
     Note:
         If a Map is found whose range is smaller than the specified block size for
@@ -377,7 +392,40 @@ def gt_set_gpu_blocksize(
         }.items():
             if f"{arg}_{dim}d" not in kwargs:
                 kwargs[f"{arg}_{dim}d"] = val
-    return sdfg.apply_transformations_once_everywhere(GPUSetBlockSize(**kwargs))
+
+    setter = GPUSetBlockSize(**kwargs)
+
+    configured_maps = 0
+    for state in sdfg.states():
+        scope_dict: Union[None, dict[Any, Any]] = None
+        cfg_id = state.parent_graph.cfg_id
+        state_id = state.block_id
+        for node in state.nodes():
+            if not isinstance(node, dace_nodes.MapEntry):
+                continue
+            if scope_dict is None:
+                scope_dict = state.scope_dict()
+            if scope_dict[node] is not None:
+                continue
+            candidate = {GPUSetBlockSize.map_entry: node}
+            setter.setup_match(
+                sdfg=sdfg,
+                cfg_id=cfg_id,
+                state_id=state_id,
+                subgraph=candidate,
+                expr_index=0,
+                override=True,
+            )
+            if setter.can_be_applied(state, 0, sdfg, False):
+                setter.apply(state, sdfg)
+                if validate_all:
+                    sdfg.validate()
+                configured_maps += 1
+
+    if validate and (not validate_all):
+        sdfg.validate()
+
+    return configured_maps
 
 
 def _make_gpu_block_parser_for(
@@ -491,6 +539,9 @@ class GPUSetBlockSize(dace_transformation.SingleStateTransformation):
         - "Over specification" is ignored, i.e. if `(32, 3, 1)` is passed as block
             size for 1 dimensional maps, then it is changed to `(32, 1, 1)`.
         - In case the Map has more than 3 dimension, normal DaCe semantic is used.
+
+    Todo:
+        - Turn this into a function.
     """
 
     _block_size_default: Final[tuple[int, int, int]] = (32, 1, 1)
@@ -584,6 +635,7 @@ class GPUSetBlockSize(dace_transformation.SingleStateTransformation):
         - If the block size of the map is already set.
         - If the map is at global scope.
         - If if the schedule of the map is correct.
+        - If launch_bounds_1d is not set then set the launch_bounds of `scan` maps to 512 to limit their register usage.
         """
         scope = graph.scope_dict()
         if scope[self.map_entry] is not None:
@@ -602,7 +654,17 @@ class GPUSetBlockSize(dace_transformation.SingleStateTransformation):
         """Modify the map as requested."""
         gpu_map: dace_nodes.Map = self.map_entry.map
         map_size = gpu_map.range.size()
-        num_map_params = len(gpu_map.params)
+        dims_to_inspect = len(map_size)
+        num_map_params = 0
+        # Order of map parameters is from outer to inner, i.e. z,y,x
+        for i, axis_size in enumerate(map_size):
+            if i < len(map_size) - 1 and map_size[i + 1] == 1:
+                assert axis_size <= 1, (
+                    "GPU thread block size setting currently does not support maps where non-leading "
+                    "dimensions have size greater than one if the next dimension has size one."
+                )
+            if axis_size != 1:
+                num_map_params += 1  # Handle 2D maps where one dimension has range 1 as 1D map
 
         # Because of a particularity of the DaCe code generator, the iteration
         #  variable that is associated to the `x` dimension of the block is the
@@ -610,24 +672,32 @@ class GPUSetBlockSize(dace_transformation.SingleStateTransformation):
         if num_map_params == 1:
             block_size = list(self.block_size_1d)
             launch_bounds = self.launch_bounds_1d
-            dims_to_inspect = 1
+            if launch_bounds is None:
+                for node in graph.scope_subgraph(
+                    self.map_entry, include_entry=False, include_exit=False
+                ):
+                    if isinstance(node, dace_nodes.NestedSDFG) and node.label.startswith("scan_"):
+                        launch_bounds = "512"  # Use high launch bound in case of scans to limit register usage and increase occupancy
         elif num_map_params == 2:
             block_size = list(self.block_size_2d)
             launch_bounds = self.launch_bounds_2d
-            dims_to_inspect = 2
         else:
             block_size = list(self.block_size_3d)
             launch_bounds = self.launch_bounds_3d
+
             # If there are more than three dimensions DaCe will condense them into
             #  the `z` dimension of the block, so we have to ignore the `z` dimension,
             #  when we modify the block sizes.
-            dims_to_inspect = 3 if num_map_params == 3 else 2
+            if num_map_params > 3:
+                dims_to_inspect = 2
 
+        # block size can only have up to three dimensions
+        assert dims_to_inspect <= 3
         # Cut down the block size.
         # TODO(phimuell): Think if it is useful to also modify the launch bounds.
         # TODO(phimuell): Also think of how to connect this with the loop blocking.
         for i in range(dims_to_inspect):
-            map_dim_idx_to_inspect = num_map_params - 1 - i
+            map_dim_idx_to_inspect = len(gpu_map.params) - 1 - i
             if (map_size[map_dim_idx_to_inspect] < block_size[i]) == True:  # noqa: E712 [true-false-comparison]  # SymPy Fancy comparison.
                 block_size[i] = map_size[map_dim_idx_to_inspect]
 
@@ -656,6 +726,8 @@ def gt_remove_trivial_gpu_maps(
         sdfg: The SDFG that we process.
         validate: Perform validation at the end of the function.
         validate_all: Perform validation also on intermediate steps.
+
+    Todo: Improve this function.
     """
 
     # First we try to promote and fuse them with other non-trivial maps.
@@ -665,9 +737,9 @@ def gt_remove_trivial_gpu_maps(
             only_gpu_maps=True,
         ),
         validate=False,
-        validate_all=False,
+        validate_all=validate_all,
     )
-    gtx_transformations.gt_simplify(sdfg, validate=validate, validate_all=validate_all)
+    gtx_transformations.gt_simplify(sdfg, validate=False, validate_all=validate_all)
 
     # Now we try to fuse them together, however, we restrict the fusion to trivial
     #  GPU map.
@@ -696,6 +768,7 @@ def gt_remove_trivial_gpu_maps(
                 return False
         return True
 
+    # TODO(phimuell): Replace this with a more performant loop.
     sdfg.apply_transformations_repeated(
         [
             gtx_transformations.MapFusionVertical(
@@ -707,10 +780,13 @@ def gt_remove_trivial_gpu_maps(
                 check_fusion_callback=restrict_to_trivial_gpu_maps,
             ),
         ],
-        validate=validate,
+        validate=False,
         validate_all=validate_all,
     )
 
+    if validate and (not validate_all):
+        sdfg.validate()
+
     return sdfg
 
 
@@ -732,6 +808,8 @@ class TrivialGPUMapElimination(dace_transformation.SingleStateTransformation):
         - This transformation should not be run on its own, instead it
             is run within the context of `gt_gpu_transformation()`.
         - This transformation must be run after the GPU Transformation.
+
+    Todo: Figuring out if this transformation is still needed.
     """
 
     only_gpu_maps = dace_properties.Property(
@@ -827,17 +905,18 @@ class TrivialGPUMapElimination(dace_transformation.SingleStateTransformation):
         #  do a temporary promotion, it is important that we do not perform the
         #  renaming. If the old symbol is still used, it is used inside a tasklet
         #  so it would show up (temporarily) as free symbol.
+        # NOTE: We use the same options as in `MapPromoter`, however, since we do
+        #   not have the list of single use data available, we have to specify it.
         org_trivial_map_params = copy.deepcopy(trivial_map.params)
         org_trivial_map_range = copy.deepcopy(trivial_map.range)
         try:
             self._promote_map(graph, replace_trivail_map_parameter=False)
             if not gtx_transformations.MapFusionVertical.can_be_applied_to(
                 sdfg=sdfg,
-                # `assume_always_shared` does not influences if Maps can be fused or not,
-                #  it only influences what happens to the intermediate storage. If we set
-                #  it we do not need to scan the entire SDFG.
                 options={
-                    "assume_always_shared": True,
+                    "only_toplevel_maps": True,
+                    "require_all_intermediates": True,
+                    "require_exclusive_intermediates": True,
                 },
                 first_map_exit=trivial_map_exit,
                 array=self.access_node,
@@ -864,15 +943,19 @@ class TrivialGPUMapElimination(dace_transformation.SingleStateTransformation):
         self._promote_map(graph)
 
         # Perform the fusing if requested.
+        #  For the selection of the option, see the `can_be_applied()` function.
         if not self.do_not_fuse:
-            # We can not specify `assume_always_shared` here, because we have to handle the
-            #  intermediate properly.
             gtx_transformations.MapFusionVertical.apply_to(
                 sdfg=sdfg,
+                options={
+                    "only_toplevel_maps": True,
+                    "require_all_intermediates": True,
+                    "require_exclusive_intermediates": True,
+                },
                 first_map_exit=trivial_map_exit,
                 array=access_node,
                 second_map_entry=second_map_entry,
-                verify=True,
+                verify=False,  # Do not rerun `can_be_applied()`.
             )
 
     def _promote_map(
@@ -906,3 +989,22 @@ class TrivialGPUMapElimination(dace_transformation.SingleStateTransformation):
         # Now copy parameter and the ranges from the second to the trivial map.
         trivial_map.params = copy.deepcopy(second_map.params)
         trivial_map.range = copy.deepcopy(second_map.range)
+
+
+def gt_gpu_apply_mempool(sdfg: dace.SDFG) -> None:
+    """Enables the stream ordered memory allocator in CUDA code generation.
+
+    It sets the `pool` flag on all transients allocated in GPU global memory.
+    The dace code genarator will handle this flag by calling `cudaMallocAsync`
+    and `cudaFreeAsync` in the CUDA code for allocation/release of the buffers.
+
+    Args:
+        sdfg: The SDFG that should be processed.
+    """
+    for _, _, desc in sdfg.arrays_recursive():
+        if (
+            isinstance(desc, dace.data.Array)
+            and desc.storage == dace.StorageType.GPU_Global
+            and desc.transient
+        ):
+            desc.pool = True
