@@ -16,7 +16,7 @@ from dace import (
     subsets as dace_subsets,
     transformation as dace_transformation,
 )
-from dace.sdfg import graph as dace_graph, nodes as dace_nodes
+from dace.sdfg import graph as dace_graph, nodes as dace_nodes, propagation as dace_propagation
 from dace.transformation import helpers as dace_helpers
 
 from gt4py.next import common as gtx_common
@@ -228,6 +228,7 @@ class LoopBlocking(dace_transformation.SingleStateTransformation):
             name=inner_label,
             ndrange=inner_range,
             schedule=dace.dtypes.ScheduleType.Sequential,
+            debuginfo=copy.copy(outer_map.debuginfo),
         )
 
         # TODO(phimuell): Investigate if we want to prevent unrolling here
@@ -835,9 +836,8 @@ class LoopBlocking(dace_transformation.SingleStateTransformation):
         # There is an invalid cache state in the SDFG, that makes the memlet
         #  propagation fail, to clear the cache we call the hash function.
         #  See: https://github.com/spcl/dace/issues/1703
-        _ = sdfg.hash_sdfg()
-        # TODO(phimuell): Use a less expensive method.
-        dace.sdfg.propagation.propagate_memlets_state(sdfg, state)
+        _ = sdfg.reset_cfg_list()
+        dace_propagation.propagate_memlets_map_scope(sdfg, state, outer_entry)
 
     def _check_if_blocking_is_favourable(
         self,
