@@ -6,6 +6,7 @@
 # Please, refer to the LICENSE file in the root directory.
 # SPDX-License-Identifier: BSD-3-Clause
 
+from enum import Enum
 from typing import Any, Optional, Sequence
 
 import dace
@@ -23,7 +24,7 @@ from gt4py.next.program_processors.runners.dace import transformations as gtx_tr
 
 def gt_remove_copy_chain(
     sdfg: dace.SDFG,
-    validate: bool = False,
+    validate: bool = True,
     validate_all: bool = False,
     single_use_data: Optional[dict[dace.SDFG, set[str]]] = None,
 ) -> Optional[int]:
@@ -58,174 +59,20 @@ def gt_remove_copy_chain(
     return result if result != 0 else None
 
 
-@dace_properties.make_properties
-class SingleStateGlobalSelfCopyElimination(dace_transformation.SingleStateTransformation):
-    """Remove global self copy.
+class CopyChainRemoverMode(Enum):
+    """Switch for `CopyChainRemover` mode.
 
-    This transformation matches the following case `(G) -> (T) -> (G)`, i.e. `G`
-    is read from and written too at the same time, however, in between is `T`
-    used as a buffer. In the example above `G` is a global memory and `T` is a
-    temporary. The transformation applies if the only incoming edge of the second
-    `G` AccessNode comes from the `T` node.
-    In case there are direct connections between the two `G` nodes, they will
-    be removed as well.
-
-    The transformation will then remove the second `G` node, all reads from the
-    second `G` node will redirected such that they use the first `G` node.
-    Furthermore, if `T` is not used downstream then also the `T` node will be
-    removed as well.
-
-    This transformation assumes that the SDFG follows rule 3 of ADR-18, which
-    guarantees that there is only a point wise dependency of the output on the
-    input.
-
-    Todo:
-        If `T` is only ready in this state, then `T` should not be created, instead
-        it should be read directly from `G`.
+    The `NONE` mode is invalid, it is there for signaling that the transformation
+    cannot be applied.
+    The `PULL` mode corresponds to the case where `a1` is transient, single-use
+    data and its full shape is copied to `a2`, thus `a1` can be replaced with `a2`.
+    The `PUSH` mode corresponds to the case where `a2` is transient, single-use
+    data and its full shape is copied from `a1`, thus `a2` can be replaced with `a1`.
     """
 
-    node_read_g = dace_transformation.PatternNode(dace_nodes.AccessNode)
-    node_tmp = dace_transformation.transformation.PatternNode(dace_nodes.AccessNode)
-    node_write_g = dace_transformation.PatternNode(dace_nodes.AccessNode)
-
-    def __init__(
-        self,
-        *args: Any,
-        **kwargs: Any,
-    ) -> None:
-        super().__init__(*args, **kwargs)
-
-    @classmethod
-    def expressions(cls) -> Any:
-        return [dace.sdfg.utils.node_path_graph(cls.node_read_g, cls.node_tmp, cls.node_write_g)]
-
-    def can_be_applied(
-        self,
-        graph: dace.SDFGState | dace.SDFG,
-        expr_index: int,
-        sdfg: dace.SDFG,
-        permissive: bool = False,
-    ) -> bool:
-        read_g = self.node_read_g
-        write_g = self.node_write_g
-        tmp_node = self.node_tmp
-        g_desc = read_g.desc(sdfg)
-        tmp_desc = tmp_node.desc(sdfg)
-
-        # NOTE: We do not check if `G` is read downstream.
-        if read_g.data != write_g.data:
-            return False
-        if g_desc.transient:
-            return False
-        if not tmp_desc.transient:
-            return False
-        if graph.scope_dict()[read_g] is not None:
-            return False
-
-        # For ease of implementation we require that only `G` defines `T`. This is
-        #  needed to ensure that `G` only depends on `G`.
-        if graph.in_degree(tmp_node) != 1:
-            return False
-
-        # Now we look at all incoming connections of the second `G` nodes, they must
-        #  either come from `T` or from the first `G` node directly.
-        for iedge in graph.in_edges(write_g):
-            if iedge.src is read_g:
-                continue
-            if iedge.src is tmp_node:
-                continue
-            return False
-
-        return True
-
-    def _is_read_downstream(
-        self,
-        start_state: dace.SDFGState,
-        sdfg: dace.SDFG,
-        data_to_look: str,
-    ) -> bool:
-        """Scans for reads to `data_to_look`.
-
-        The function will go through states that are reachable from `start_state`
-        (including) and test if there is a read to the data container `data_to_look`.
-        It will return `True` the first time it finds such a node.
-        It is important that the matched nodes, i.e. `self.node_{read_g, write_g, tmp}`
-        are ignored.
-
-        Args:
-            start_state: The state where the scanning starts.
-            sdfg: The SDFG on which we operate.
-            data_to_look: The data that we want to look for.
-
-        Todo:
-            Port this function to use DaCe pass pipeline.
-        """
-        read_g: dace_nodes.AccessNode = self.node_read_g
-        write_g: dace_nodes.AccessNode = self.node_write_g
-        tmp_node: dace_nodes.AccessNode = self.node_tmp
-
-        # TODO(phimuell): Run the `StateReachability` pass in a pipeline and use
-        #  the `_pipeline_results` member to access the data.
-        return gtx_transformations.utils.is_accessed_downstream(
-            start_state=start_state,
-            sdfg=sdfg,
-            reachable_states=None,
-            data_to_look=data_to_look,
-            nodes_to_ignore={read_g, write_g, tmp_node},
-        )
-
-    def apply(
-        self,
-        graph: dace.SDFGState | dace.SDFG,
-        sdfg: dace.SDFG,
-    ) -> None:
-        read_g: dace_nodes.AccessNode = self.node_read_g
-        write_g: dace_nodes.AccessNode = self.node_write_g
-        tmp_node: dace_nodes.AccessNode = self.node_tmp
-
-        # First check if the `T` node is still needed. The node is needed if the data
-        #  is referenced downstream or if there are edges other than to the second
-        #  `G` node.
-        if not all(oedge.dst is write_g for oedge in graph.out_edges(tmp_node)):
-            tmp_node_is_still_needed = True
-        elif self._is_read_downstream(start_state=graph, sdfg=sdfg, data_to_look=tmp_node.data):
-            tmp_node_is_still_needed = True
-        else:
-            tmp_node_is_still_needed = False
-
-        # Redirect the reads from the second `G` node such that they now go through
-        #  the first `G` node. Because the names are the same there is no special
-        #  handling needed.
-        # NOTE: There are no writes to the second `G` nodes beside directly the one
-        #  coming from the first `G` node or the `T` node. Thus nothing to do.
-        for wg_oedge in list(graph.out_edges(write_g)):
-            graph.add_edge(
-                read_g,
-                wg_oedge.src_conn,
-                wg_oedge.dst,
-                wg_oedge.dst_conn,
-                wg_oedge.data,
-            )
-            graph.remove_edge(wg_oedge)
-
-        # Now we remove the second `G` node as it is no longer needed.
-        graph.remove_node(write_g)
-
-        # If the `T` is no longer needed then remove it. If the first node has become
-        #  isolated, also remove it.
-        if not tmp_node_is_still_needed:
-            graph.remove_node(tmp_node)
-
-            if graph.degree(read_g) == 0:
-                graph.remove_node(read_g)
-
-            # If it fails then `T` is still used in a parallel controlflow path, for
-            #  example in `if` branches.
-            try:
-                sdfg.remove_data(tmp_node.data, validate=True)
-            except ValueError as e:
-                if not str(e).startswith(f"Cannot remove data descriptor {tmp_node.data}:"):
-                    raise
+    NONE = 0
+    PULL = 1
+    PUSH = 2
 
 
 @dace_properties.make_properties
@@ -246,9 +93,17 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
     requirements before it can be applied:
     - Through the merging of `A1` and `A2` no cycles are created.
     - `A1` can not be used anywhere else.
-    - `A1` is a transient and must have the same dimensionality than `A2`.
+    - `A1` is a transient and must have the same dimensionality as `A2`.
     - `A1` is fully read by `A2`.
 
+    This transformation corresponds to the `PULL` mode of `CopyChainRemoverMode`.
+    In a second iteraton, the transformation was extended to support the mirror
+    case (`PUSH`), where A2 is removed if the following requirements are satisfied:
+    - Through the merging of `A1` and `A2` no cycles are created.
+    - `A2` can not be used anywhere else.
+    - `A2` is a transient and must have the same dimensionality as `A1`.
+    - `A2` is fully written by `A1`.
+
     Notes:
         The transformation assumes that the domain inference adjusted the ranges of
         the maps such that, in case they write into a transient, the full shape of
@@ -307,13 +162,8 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
         a1_desc = a1.desc(sdfg)
         a2_desc = a2.desc(sdfg)
 
-        # We remove `a1` so it must be a transient and used only once.
-        if not a1_desc.transient:
-            return False
-        if not self.is_single_use_data(sdfg, a1):
-            return False
-
         # This avoids that we have to modify the subsets in a fancy way.
+        # TODO(phimuell): Lift this limitation.
         if len(a1_desc.shape) != len(a2_desc.shape):
             return False
 
@@ -355,19 +205,10 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
         if dst_subset is None:
             return False
 
-        # Checking if the whole array is read.
-        # NOTE: The main benefit of requiring that the whole array is read is that we
-        #   do not have to adjust maps.
-        # NOTE: In previous versions there was an ad hoc rule, to bypass the "full
-        #   read rule". However, it caused problems, so it was removed.
-        # TODO: We have to improve this test, because sometimes the expressions are
-        #   so complex that without information about relations, such as
-        #   `vertical_start <= vertical_end` it is not possible to prove this check.
-        a1_range = dace_sbs.Range.from_array(a1_desc)
-        if not src_subset.covers(a1_range):
+        if self._get_copy_chain_mode(sdfg, graph) == CopyChainRemoverMode.NONE:
             return False
 
-        # We have to ensure that no cycle is created through the removal of `a1`.
+        # We have to ensure that no cycle is created through the removal of one node.
         #  For this we have to ensure that there is no connection, beside the direct
         #  one between `a1` and `a2`.
         # NOTE: We only check the outgoing edges of `a1`, it is not needed to also
@@ -379,10 +220,10 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
         ):
             return False
 
-        # NOTE: In case `a2` is a non transient we do not have to check if it is read
-        #   or written to somewhere else in this state. The reason is that ADR18
-        #   guarantees us that everything is point wise, therefore `a1` is never
-        #   used as double buffer.
+        # NOTE: In case the node we keep is a non-transient we do not have to
+        #   check if it is read or written to somewhere else in this state.
+        #   The reason is that ADR18 guarantees that every access is point-wise,
+        #   therefore the temporary we remove is never used as double buffer.
         return True
 
     def is_single_use_data(
@@ -407,6 +248,7 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
             oedge for oedge in graph.out_edges(a1) if oedge.dst is a2
         )
         a1_to_a2_memlet: dace.Memlet = a1_to_a2_edge.data
+        a1_to_a2_src_subset: dace_sbs.Range = a1_to_a2_memlet.get_src_subset(a1_to_a2_edge, graph)
         a1_to_a2_dst_subset: dace_sbs.Range = a1_to_a2_memlet.get_dst_subset(a1_to_a2_edge, graph)
 
         # Note that it is possible that `a1` is connected to the same node multiple
@@ -416,31 +258,53 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
         #  counter example would be a Map with different connector names.
         reconfigured_neighbour: set[tuple[dace_nodes.Node, Optional[str]]] = set()
 
-        # Now we compose the new subset.
-        #  We build on the fact that we have ensured that the whole array `a1` is
-        #  copied into `a2`. Thus the destination of the original source, i.e.
-        #  whatever write into `a1`, is just offset by the beginning of the range
-        #  `a1` writes into `a2`.
-        #       (s1) ------[c:d]-> (A1) -[0:N]------[a:b]-> (A2)
-        #       (s1) ---------[(a + c):(a + c + (d - c))]-> (A2)
-        #  Thus the offset is simply given by `a`, the start where `a1` is written into
-        #  `a2`.
-        #  NOTE: If we ever allow the that `a1` is not fully read, then we would have
-        #   to modify this computation slightly.
-        a2_offsets: Sequence[dace_sym.SymExpr] = a1_to_a2_dst_subset.min_element()
+        #  In this section, `old_node` refers to the node, one of `a1` or `a2`,
+        #  which will be replaced by the other one. The node which is kept is
+        #  referred to as `new_node`.
+        new_node: dace_nodes.AccessNode
+        old_node: dace_nodes.AccessNode
+        new_node_offsets: Sequence[dace_sym.SymbolicType]
+        copy_chain_mode = self._get_copy_chain_mode(sdfg, graph)
+        if copy_chain_mode == CopyChainRemoverMode.PULL:
+            old_node = a1
+            new_node = a2
+            # Now we compose the new subset.
+            #  For the `PULL` mode, we build on the fact that we have ensured that
+            #  the whole array `old_node` (`a1`) is copied into `new_node` (`a2`).
+            #  Thus the destination of the original source, i.e. whatever writes
+            #  into `a1`, is just offset by the beginning of the range `a1` writes
+            #  into `a2`.
+            #       (s1) ------[c:d]-> (A1) -[0:N]------[a:b]-> (A2)
+            #       (s1) ---------[(a + c):(a + c + (d - c))]-> (A2)
+            #  Thus the offset is simply given by `a`, the start index where `a1`
+            #  is written into `a2`.
+            #  NOTE: If we ever allow the that `a1` is not fully read, then we would
+            #   have to modify this computation slightly.
+            new_node_offsets = a1_to_a2_dst_subset.min_element()
+        else:
+            assert copy_chain_mode == CopyChainRemoverMode.PUSH
+            old_node = a2
+            new_node = a1
+            # We compose the new subset in a similar way, but using the index where
+            #  `a2` starts reading from `a1`.
+            new_node_offsets = a1_to_a2_src_subset.min_element()
 
         # Handle the producer side of things.
-        for producer_edge in list(graph.in_edges(a1)):
+        for producer_edge in list(graph.in_edges(old_node)):
             producer: dace_nodes.Node = producer_edge.src
             producer_conn = producer_edge.src_conn
+            if producer is new_node:
+                assert producer_edge is a1_to_a2_edge
+                assert copy_chain_mode == CopyChainRemoverMode.PUSH
+                continue
             new_producer_edge = gtx_transformations.utils.reroute_edge(
                 is_producer_edge=True,
                 current_edge=producer_edge,
-                ss_offset=a2_offsets,
+                ss_offset=new_node_offsets,
                 state=graph,
                 sdfg=sdfg,
-                old_node=a1,
-                new_node=a2,
+                old_node=old_node,
+                new_node=new_node,
             )
             if (producer, producer_conn) not in reconfigured_neighbour:
                 gtx_transformations.utils.reconfigure_dataflow_after_rerouting(
@@ -448,28 +312,29 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
                     new_edge=new_producer_edge,
                     sdfg=sdfg,
                     state=graph,
-                    ss_offset=a2_offsets,
-                    old_node=a1,
-                    new_node=a2,
+                    ss_offset=new_node_offsets,
+                    old_node=old_node,
+                    new_node=new_node,
                 )
                 reconfigured_neighbour.add((producer, producer_conn))
 
         # Handle the consumer side of things, as they now have to read from `a2`.
         #  It is important that the offset is still the same.
-        for consumer_edge in list(graph.out_edges(a1)):
+        for consumer_edge in list(graph.out_edges(old_node)):
             consumer: dace_nodes.Node = consumer_edge.dst
             consumer_conn = consumer_edge.dst_conn
-            if consumer is a2:
+            if consumer is new_node:
                 assert consumer_edge is a1_to_a2_edge
+                assert copy_chain_mode == CopyChainRemoverMode.PULL
                 continue
             new_consumer_edge = gtx_transformations.utils.reroute_edge(
                 is_producer_edge=False,
                 current_edge=consumer_edge,
-                ss_offset=a2_offsets,
+                ss_offset=new_node_offsets,
                 state=graph,
                 sdfg=sdfg,
-                old_node=a1,
-                new_node=a2,
+                old_node=old_node,
+                new_node=new_node,
             )
             if (consumer, consumer_conn) not in reconfigured_neighbour:
                 gtx_transformations.utils.reconfigure_dataflow_after_rerouting(
@@ -477,23 +342,78 @@ class CopyChainRemover(dace_transformation.SingleStateTransformation):
                     new_edge=new_consumer_edge,
                     sdfg=sdfg,
                     state=graph,
-                    ss_offset=a2_offsets,
-                    old_node=a1,
-                    new_node=a2,
+                    ss_offset=new_node_offsets,
+                    old_node=old_node,
+                    new_node=new_node,
                 )
                 reconfigured_neighbour.add((consumer, consumer_conn))
 
-        # After the rerouting we have to delete the `a1` data node and descriptor,
+        # After the rerouting we have to delete the transient data node and descriptor,
         #  this will also remove all the old edges.
-        graph.remove_node(a1)
-        sdfg.remove_data(a1.data, validate=False)
+        graph.remove_node(old_node)
+        sdfg.remove_data(old_node.data, validate=False)
 
-        # We will now propagate the strides starting from the access nodes `a2`.
-        #  Essentially, this will replace the strides from `a1` with the ones of
-        #  `a2`. We do it outside to make sure that we do not forget a case and
-        #  that we propagate the change into every NestedSDFG only once.
+        # We will now propagate the strides starting from the access node we keep.
+        #  Essentially, this will replace the strides from `old_node` with the ones
+        #  of `new_node`. We do it outside to make sure that we do not forget a
+        #  case and that we propagate the change into every NestedSDFG only once.
         gtx_transformations.gt_propagate_strides_from_access_node(
             sdfg=sdfg,
             state=graph,
-            outer_node=a2,
+            outer_node=new_node,
         )
+
+    def _get_copy_chain_mode(self, sdfg: dace.SDFG, state: dace.SDFG) -> CopyChainRemoverMode:
+        """
+        Helper function to detect whether the `CopyChainRemover` tranbsformation
+        should be applied in `PULL` or `PUSH` mode, for details see `CopyChainRemoverMode`.
+        """
+        a1: dace_nodes.AccessNode = self.node_a1
+        a2: dace_nodes.AccessNode = self.node_a2
+
+        a1_desc = a1.desc(sdfg)
+        a1_range = dace_sbs.Range.from_array(a1_desc)
+
+        a2_desc = a2.desc(sdfg)
+        a2_range = dace_sbs.Range.from_array(a2_desc)
+
+        # There shall only be one edge connecting `a1` and `a2`.
+        #  We even strengthen this requirement by not checking for the node `a2`,
+        #  but for the data.
+        connecting_edges = [
+            oedge
+            for oedge in state.out_edges(a1)
+            if isinstance(oedge.dst, dace_nodes.AccessNode) and (oedge.dst.data == a2.data)
+        ]
+        assert len(connecting_edges) == 1
+
+        # The full array `a1` is copied into `a2`. Note that it is allowed, that
+        #  `a2` is bigger than `a1`, it is just important that everything that was
+        #  written into `a1` is also accessed.
+        connecting_edge = connecting_edges[0]
+        assert connecting_edge.dst is a2
+        connecting_memlet = connecting_edge.data
+
+        # If the destination or the source subset of the connection is not fully
+        #  specified, we do not apply.
+        src_subset = connecting_memlet.get_src_subset(connecting_edge, state)
+        assert src_subset is not None
+        dst_subset = connecting_memlet.get_dst_subset(connecting_edge, state)
+        assert dst_subset is not None
+
+        # Checking if the whole array is read.
+        # NOTE: The main benefit of requiring that the whole array is read is that we
+        #   do not have to adjust maps.
+        # NOTE: In previous versions there was an ad hoc rule, to bypass the "full
+        #   read rule". However, it caused problems, so it was removed.
+        # TODO: We have to improve this test, because sometimes the expressions are
+        #   so complex that without information about relations, such as
+        #   `vertical_start <= vertical_end` it is not possible to prove this check.
+        if a1_desc.transient and self.is_single_use_data(sdfg, a1) and src_subset.covers(a1_range):
+            return CopyChainRemoverMode.PULL
+
+        # Checking if the whole array is written.
+        if a2_desc.transient and self.is_single_use_data(sdfg, a2) and dst_subset.covers(a2_range):
+            return CopyChainRemoverMode.PUSH
+
+        return CopyChainRemoverMode.NONE
