@@ -6,6 +6,8 @@
 # Please, refer to the LICENSE file in the root directory.
 # SPDX-License-Identifier: BSD-3-Clause
 
+"""Utilities to translate from Python types to GT4Py type specifications."""
+
 from __future__ import annotations
 
 import builtins
@@ -19,12 +21,13 @@ import numpy as np
 import numpy.typing as npt
 
 from gt4py._core import definitions as core_defs
-from gt4py.eve import extended_typing as xtyping
+from gt4py.eve import extended_typing as xtyping, utils as eve_utils
 from gt4py.next import common
 from gt4py.next.type_system import type_info, type_specifications as ts
 
 
 def get_scalar_kind(dtype: npt.DTypeLike) -> ts.ScalarKind:
+    """Get the GT4Py scalar kind for a given NumPy dtype."""
     # make int & float precision platform independent.
     dt: np.dtype
     if dtype is builtins.int:
@@ -37,6 +40,9 @@ def get_scalar_kind(dtype: npt.DTypeLike) -> ts.ScalarKind:
         except TypeError as err:
             raise ValueError(f"Invalid scalar type definition ('{dtype}').") from err
 
+    if dt.hasobject:
+        raise ValueError("Object dtypes are not supported.")
+
     if dt.shape == () and dt.fields is None:
         match dt:
             case np.bool_:
@@ -51,16 +57,16 @@ def get_scalar_kind(dtype: npt.DTypeLike) -> ts.ScalarKind:
         raise ValueError(f"Non-trivial dtypes like '{dtype}' are not yet supported.")
 
 
-def from_type_hint(
+def canonicalize_type_hint(
     type_hint: Any,
     *,
     globalns: Optional[dict[str, Any]] = None,
     localns: Optional[dict[str, Any]] = None,
-) -> ts.TypeSpec:
-    recursive_make_symbol = functools.partial(from_type_hint, globalns=globalns, localns=localns)
-    extra_args: list = []
-
-    # ForwardRef
+) -> tuple[Any, tuple[Any, ...], tuple[Any, ...]]:
+    """
+    Canonicalize python type annotations as a tuple of (canonical_type, type_args, annotated_extra_args).
+    """
+    # Canonicalize 'ForwardRef()' annotations
     if isinstance(type_hint, str):
         type_hint = ForwardRef(type_hint)
     if isinstance(type_hint, ForwardRef):
@@ -71,7 +77,8 @@ def from_type_hint(
                 f"Type annotation '{type_hint}' has undefined forward references."
             ) from error
 
-    # Annotated
+    # Cannonicalize 'Annotated' annotations
+    extra_args = []
     if typing.get_origin(type_hint) is typing.Annotated:
         type_hint, *extra_args = typing.get_args(type_hint)
         if not isinstance(
@@ -83,16 +90,30 @@ def from_type_hint(
     canonical_type = typing.get_origin(type_hint) or type_hint
     args = typing.get_args(type_hint)
 
-    match canonical_type:
-        case type() as t if issubclass(t, (bool, int, float, np.generic, str)):
-            return ts.ScalarType(kind=get_scalar_kind(type_hint))
+    return canonical_type, args, tuple(extra_args)
 
+
+@eve_utils.optional_lru_cache(maxsize=None, typed=False)
+def from_type_hint(
+    type_hint: Any,
+    *,
+    globalns: Optional[dict[str, Any]] = None,
+    localns: Optional[dict[str, Any]] = None,
+) -> ts.TypeSpec:
+    """Convert any kind of Python type hint to a GT4Py TypeSpec."""
+    from_type_hint_same_ns = functools.partial(from_type_hint, globalns=globalns, localns=localns)
+
+    canonical_type, args, extra_args = canonicalize_type_hint(
+        type_hint, globalns=globalns, localns=localns
+    )
+
+    match canonical_type:
         case builtins.tuple:
             if not args:
                 raise ValueError(f"Tuple annotation '{type_hint}' requires at least one argument.")
             if Ellipsis in args:
                 raise ValueError(f"Unbound tuples '{type_hint}' are not allowed.")
-            tuple_types = [recursive_make_symbol(arg) for arg in args]
+            tuple_types = [from_type_hint_same_ns(arg) for arg in args]
             assert all(isinstance(elem, ts.DataType) for elem in tuple_types)
             return ts.TupleType(types=tuple_types)
 
@@ -115,7 +136,7 @@ def from_type_hint(
                 raise ValueError(f"Invalid field dimensions '{dim_arg}'.")
 
             try:
-                dtype = recursive_make_symbol(dtype_arg)
+                dtype = from_type_hint_same_ns(dtype_arg)
             except ValueError as error:
                 raise ValueError(
                     f"Field dtype argument must be a scalar type (got '{dtype_arg}')."
@@ -130,7 +151,7 @@ def from_type_hint(
 
             try:
                 arg_types, return_type = args
-                new_args = [recursive_make_symbol(arg) for arg in arg_types]
+                new_args = [from_type_hint_same_ns(arg) for arg in arg_types]
                 assert all(isinstance(arg, ts.DataType) for arg in new_args)
             except Exception as error:
                 raise ValueError(f"Invalid callable annotations in '{type_hint}'.") from error
@@ -139,12 +160,12 @@ def from_type_hint(
             if len(kwargs_info) != 1:
                 raise ValueError(f"Invalid callable annotations in '{type_hint}'.")
             kwargs = {
-                arg: recursive_make_symbol(arg_type)
+                arg: from_type_hint_same_ns(arg_type)
                 for arg, arg_type in kwargs_info[0].data.items()
             }
             assert all(isinstance(val, (ts.DataType, ts.DeferredType)) for val in kwargs.values())
 
-            returns = recursive_make_symbol(return_type)
+            returns = from_type_hint_same_ns(return_type)
             assert isinstance(returns, (ts.DataType, ts.DeferredType, ts.VoidType))
 
             # TODO(tehrengruber): print better error when no return type annotation is given
@@ -154,6 +175,28 @@ def from_type_hint(
                 kw_only_args={},  # TODO
                 returns=returns,
             )
+
+        case builtins.type if args:
+            # This case matches 'type[Foo]' (where the 'Foo' type is stored in args[0])
+            python_type = args[0]
+
+            constructed_type_spec = from_type_hint_same_ns(python_type)
+            if not isinstance(constructed_type_spec, ts.ScalarType):
+                raise TypeError(f"Cannot create constructor for type: {constructed_type_spec}")
+
+            return ts.ConstructorType(
+                definition=ts.FunctionType(
+                    pos_only_args=[ts.DeferredType(constraint=ts.ScalarType)],
+                    pos_or_kw_args={},
+                    kw_only_args={},
+                    returns=constructed_type_spec,
+                )
+            )
+
+        case type() if issubclass(canonical_type, (*core_defs.SCALAR_TYPES, str)):
+            # This case matches 'int', 'float', etc. used as annotations
+            return ts.ScalarType(kind=get_scalar_kind(canonical_type))
+
     raise ValueError(f"'{type_hint}' type is not supported.")
 
 
@@ -169,9 +212,9 @@ class UnknownPythonObject(ts.TypeSpec):
 
 
 def from_value(value: Any) -> ts.TypeSpec:
+    """Make a symbol node from a Python value."""
     # TODO(tehrengruber): use protocol from gt4py.next.common when available
     #  instead of importing from the embedded implementation
-    """Make a symbol node from a Python value."""
     # TODO(tehrengruber): What we expect here currently is a GTCallable. Maybe
     #  we should check for the protocol in the future?
     if hasattr(value, "__gt_type__"):
@@ -206,15 +249,13 @@ def from_value(value: Any) -> ts.TypeSpec:
         elems = [from_value(el) for el in value]
         assert all(isinstance(elem, ts.DataType) for elem in elems)
         return ts.TupleType(types=elems)
-    elif isinstance(value, types.ModuleType):
+    elif isinstance(value, (types.ModuleType, eve_utils.FrozenNamespace)):
         return UnknownPythonObject(value)
     else:
         type_ = xtyping.infer_type(value, annotate_callable_kwargs=True)
         symbol_type = from_type_hint(type_)
 
-    if isinstance(symbol_type, (ts.DataType, ts.OffsetType, ts.DimensionType)) or (
-        isinstance(symbol_type, ts.CallableType) and isinstance(symbol_type, ts.TypeSpec)
-    ):
+    if isinstance(symbol_type, (ts.DataType, ts.CallableType, ts.OffsetType, ts.DimensionType)):
         return symbol_type
     else:
         raise ValueError(f"Impossible to map '{value}' value to a 'Symbol'.")
